from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.helpers.panther_snyk_helpers import snyk_alert_context
from pypanther.log_types import PantherLogType

snyk_service_account_change_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Snyk Org Service Account ADMIN role ( CRIT )",
        ExpectedResult=True,
        Log={
            "content": {
                "role": "ADMIN",
                "rolePublicId": "d8999999-aaaa-4444-9fff-955555555555",
                "serviceAccountPublicId": "9ddddddd-4444-4111-9eee-188888888888",
            },
            "created": "2023-04-05 22:22:57.488",
            "event": "org.service_account.create",
            "groupId": "8fffffff-1555-4444-b000-b55555555555",
            "orgId": "21111111-a222-4eee-8ddd-a99999999999",
            "userId": "05555555-3333-4ddd-8ccc-755555555555",
        },
    ),
    PantherRuleTest(
        Name="Snyk Group Service Account ADMIN role ( CRIT )",
        ExpectedResult=True,
        Log={
            "content": {
                "role": {
                    "created": "1999-01-22T16:02:53.848Z",
                    "description": "Admin",
                    "groupPermissions": [
                        "group.read",
                        "group.edit",
                        "group.iac.settings.read",
                        "group.iac.settings.edit",
                        "group.report.read",
                        "group.org.add",
                        "group.org.remove",
                        "group.org.list",
                        "group.user.add",
                        "group.user.read",
                        "group.user.remove",
                        "group.user.delete",
                        "group.user.role.edit",
                        "group.user.provision",
                        "group.service_account.create",
                        "group.service_account.read",
                        "group.service_account.edit",
                        "group.service_account.delete",
                        "group.audit.read",
                        "group.settings.read",
                        "group.settings.edit",
                        "group.notification_settings.read",
                        "group.notification_settings.edit",
                        "group.policy.read",
                        "group.policy.create",
                        "group.policy.edit",
                        "group.policy.delete",
                        "group.tag.read",
                        "group.tag.create",
                        "group.tag.delete",
                        "group.flags.read",
                        "group.flags.edit",
                        "group.settings.request_access.read",
                        "group.settings.request_access.edit",
                        "group.sso.read",
                        "group.sso.edit",
                        "group.sso.remove",
                        "group.roles.read",
                        "group.roles.create",
                        "group.roles.edit",
                        "group.roles.remove",
                    ],
                    "id": 1,
                    "modified": "1999-01-07T12:36:51.716Z",
                    "name": "Group Admin",
                    "orgPermissions": [
                        "org.read",
                        "org.create",
                        "org.edit",
                        "org.delete",
                        "org.report.read",
                        "org.project.create",
                        "org.project.read",
                        "org.project.edit",
                        "org.project.delete",
                        "org.project.status",
                        "org.project.test",
                        "org.project.move",
                        "org.project.integration.edit",
                        "org.project.jira.issue.read",
                        "org.project.jira.issue.create",
                        "org.project.ignore.create",
                        "org.project.ignore.read",
                        "org.project.ignore.edit",
                        "org.project.ignore.delete",
                        "org.project.pr.create",
                        "org.project.pr.skip",
                        "org.project.attributes.edit",
                        "org.project.tag.edit",
                        "org.service_account.create",
                        "org.service_account.read",
                        "org.service_account.edit",
                        "org.service_account.delete",
                        "org.user.add",
                        "org.user.invite",
                        "org.user.manage",
                        "org.user.read",
                        "org.user.delete",
                        "org.user.leave",
                        "org.user.provision",
                        "org.integration.read",
                        "org.integration.edit",
                        "org.package.test",
                        "org.billing.read",
                        "org.billing.edit",
                        "org.entitlements.read",
                        "org.audit_log.read",
                        "org.flags.read",
                        "org.flags.edit",
                        "org.outbound_webhook.read",
                        "org.outbound_webhook.create",
                        "org.outbound_webhook.delete",
                        "org.app.read",
                        "org.app.install",
                        "org.app.create",
                        "org.app.delete",
                        "org.app.edit",
                        "org.cloud_environments.read",
                        "org.cloud_environments.create",
                        "org.cloud_environments.delete",
                        "org.cloud_environments.edit",
                        "org.cloud_scans.read",
                        "org.cloud_scans.create",
                        "org.cloud_resources.read",
                        "org.cloud_artifacts.read",
                        "org.cloud_artifacts.create",
                        "org.cloud_custom_rules.read",
                        "org.cloud_custom_rules.create",
                        "org.cloud_custom_rules.edit",
                        "org.cloud_custom_rules.delete",
                    ],
                    "publicId": "89999999-feee-4333-8ccc-5aaaaaaaaaaa",
                    "role": "ADMIN",
                    "userPermissions": ["user.activate", "user.deactivate"],
                },
                "serviceAccountPublicId": "98888888-1ccc-4aaa-9fff-d66666666666",
            },
            "created": "2021-02-09 17:46:16.622",
            "event": "group.service_account.create",
            "groupId": "8fffffff-1555-4444-b000-b55555555555",
            "userId": "05555555-3333-4ddd-8ccc-755555555555",
        },
    ),
    PantherRuleTest(
        Name="Snyk Group Service Account Create - Viewer Role ( HIGH )",
        ExpectedResult=True,
        Log={
            "content": {
                "role": {
                    "created": "1999-01-22T16:02:53.848Z",
                    "description": "Viewer",
                    "groupPermissions": ["group.tag.read"],
                    "id": 3,
                    "modified": "1999-02-01T12:36:51.728Z",
                    "name": "Group Viewer",
                    "orgPermissions": ["org.read"],
                    "publicId": "24444444-addd-4ddd-a444-0ddddddddddd",
                    "role": "VIEWER",
                    "userPermissions": [],
                },
                "serviceAccountPublicId": "06666666-9eee-4ccc-8999-4bbbbbbbbbbb",
            },
            "created": "2021-01-08 17:13:01.355",
            "event": "group.service_account.create",
            "groupId": "8fffffff-1555-4444-b000-b55555555555",
            "userId": "05555555-3333-4ddd-8ccc-755555555555",
        },
    ),
    PantherRuleTest(
        Name="Snyk Service Account Edit ( MEDIUM )",
        ExpectedResult=True,
        Log={
            "groupId": "8fffffff-1555-4444-b000-b55555555555",
            "userId": "05555555-3333-4ddd-8ccc-755555555555",
            "event": "group.service_account.edit",
            "content": {
                "after": {"name": "test-SA-after"},
                "before": {"name": "test-SA-before"},
                "serviceAccountPublicId": "41111111-3fff-4eee-b111-6bbbbbbbbbbb",
            },
            "created": "2023-02-12T23:57:35.522Z",
        },
    ),
    PantherRuleTest(
        Name="Snyk Group SSO Membership sync",
        ExpectedResult=False,
        Log={
            "content": {},
            "created": "2023-03-15 13:13:13.133",
            "event": "group.sso.membership.sync",
            "groupId": "8fffffff-1555-4444-b000-b55555555555",
        },
    ),
]


class SnykServiceAccountChange(PantherRule):
    RuleID = "Snyk.ServiceAccount.Change-prototype"
    DisplayName = "Snyk Service Account Change"
    LogTypes = [PantherLogType.Snyk_GroupAudit, PantherLogType.Snyk_OrgAudit]
    Tags = ["Snyk"]
    Severity = PantherSeverity.High
    Description = "Detects when Snyk Service Accounts are changed\n"
    Runbook = "These actions in the Snyk Audit logs indicate that a ServiceAccount has been created/deleted/modified.\nService Accounts are system user accounts with an API token associated to it in place of standard user credentials. All events where the Service Account's role is ADMIN have CRITICAL severity Deletion events are marked with HIGH severity Creation events are marked with HIGH severity Edit events are marked with MEDIUM Severity\n"
    Reference = "https://docs.snyk.io/snyk-admin/service-accounts"
    SummaryAttributes = ["event"]
    Tests = snyk_service_account_change_tests
    ACTIONS = [
        "group.service_account.create",
        "group.service_account.delete",
        "group.service_account.edit",
        "org.service_account.create",
        "org.service_account.delete",
        "org.service_account.edit",
        "org.service_account.membership.upsert",
    ]

    def rule(self, event):
        action = deep_get(event, "event", default="<NO_EVENT>")
        return action in self.ACTIONS

    def title(self, event):
        group_or_org = "<GROUP_OR_ORG>"
        crud_operation = "<NO_OPERATION>"
        action = deep_get(event, "event", default="<NO_EVENT>")
        if "." in action:
            group_or_org = action.split(".")[0].title()
            crud_operation = action.split(".")[-1].title()
        return f"Snyk: [{group_or_org}] Service Account [{crud_operation}] performed by [{deep_get(event, 'userId', default='<NO_USERID>')}]"

    def alert_context(self, event):
        a_c = snyk_alert_context(event)
        role = deep_get(event, "content", "role", "role", default=None)
        if not role:
            role = deep_get(event, "content", "role", default=None)
        if role:
            a_c["role_permission"] = role
        return a_c

    def dedup(self, event):
        return f"{deep_get(event, 'userId', default='<NO_USERID>')}{deep_get(event, 'orgId', default='<NO_ORGID>')}{deep_get(event, 'groupId', default='<NO_GROUPID>')}{deep_get(event, 'event', default='<NO_EVENT>')}"

    def severity(self, event):
        action = deep_get(event, "event", default="<NO_EVENT>")
        role = deep_get(event, "content", "role", "role", default=None)
        if not role:
            role = deep_get(event, "content", "role", default=None)
        if all(
            [
                role == "ADMIN",
                action.endswith((".service_account.create", ".service_account.delete")),
            ]
        ):
            return "CRITICAL"
        if action.endswith((".service_account.create", ".service_account.delete")):
            return "HIGH"
        return "MEDIUM"



import ipaddress
from typing import List

import pypanther.helpers.panther_event_type_helpers as event_type
from pypanther.base import PantherDataModel, PantherDataModelMapping
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.log_types import PantherLogType


def get_event_type(event):
    # currently, only tracking a few event types
    if (
        event.get("eventName") == "ConsoleLogin"
        and deep_get(event, "userIdentity", "type") == "IAMUser"
    ):
        if deep_get(event, "responseElements", "ConsoleLogin") == "Failure":
            return event_type.FAILED_LOGIN
        if deep_get(event, "responseElements", "ConsoleLogin") == "Success":
            return event_type.SUCCESSFUL_LOGIN
    if event.get("eventName") == "CreateUser":
        return event_type.USER_ACCOUNT_CREATED
    if event.get("eventName") == "CreateAccountResult":
        return event_type.ACCOUNT_CREATED
    return None


def load_ip_address(event):
    """
    CloudTrail occasionally sets non-IPs in the sourceIPAddress field.
    This method ensures that either an IPv4 or IPv6 address is always returned.
    """
    source_ip = event.get("sourceIPAddress")
    if not source_ip:
        return None
    try:
        ipaddress.IPv4Address(source_ip)
    except ipaddress.AddressValueError:
        try:
            ipaddress.IPv6Address(source_ip)
        except ipaddress.AddressValueError:
            return None
    return source_ip


class StandardAWSCloudTrail(PantherDataModel):
    DataModelID: str = "Standard.AWS.CloudTrail"
    DisplayName: str = "AWS CloudTrail"
    Enabled: bool = True
    LogTypes: List[str] = [PantherLogType.AWS_CloudTrail]
    Mappings: List[PantherDataModelMapping] = [
        PantherDataModelMapping(Name="actor_user", Path="$.userIdentity..userName"),
        PantherDataModelMapping(Name="event_type", Method=get_event_type),
        PantherDataModelMapping(Name="source_ip", Method=load_ip_address),
        PantherDataModelMapping(Name="user_agent", Path="userAgent"),
        PantherDataModelMapping(Name="user", Path="$.responseElements.user.userName"),
        PantherDataModelMapping(Name="user_account_id", Path="$.responseElements.user.userId"),
    ]



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.log_types import PantherLogType

teleport_lock_created_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="A Lock was created",
        ExpectedResult=True,
        Log={
            "cluster_name": "teleport.example.com",
            "code": "TLK00I",
            "ei": 0,
            "event": "lock.created",
            "expires": "0001-01-01T00:00:00Z",
            "name": "88888888-4444-4444-4444-222222222222",
            "target": {"user": "user-to-disable"},
            "time": "2023-09-21T00:00:00.000000Z",
            "uid": "88888888-4444-4444-4444-222222222222",
            "updated_by": "max.mustermann@example.com",
            "user": "max.mustermann@example.com",
        },
    )
]


class TeleportLockCreated(PantherRule):
    RuleID = "Teleport.LockCreated-prototype"
    DisplayName = "A Teleport Lock was created"
    LogTypes = [PantherLogType.Gravitational_TeleportAudit]
    Tags = ["Teleport"]
    Severity = PantherSeverity.Info
    Description = "A Teleport Lock was created"
    Reference = "https://goteleport.com/docs/management/admin/"
    Runbook = "A Teleport Lock was created; this is an unusual administrative action. Investigate to understand why a Lock was created.\n"
    SummaryAttributes = ["event", "code", "time", "identity"]
    Tests = teleport_lock_created_tests

    def rule(self, event):
        return event.get("event") == "lock.created"

    def title(self, event):
        return f"A Teleport Lock was created by {event.get('updated_by', '<UNKNOWN_UPDATED_BY>')} to Lock out user {event.get('target', {}).get('user', '<UNKNOWN_USER>')} on [{event.get('cluster_name', '<UNKNOWN_CLUSTER>')}]"



from typing import List

from pypanther.base import PantherDataModel, PantherDataModelMapping
from pypanther.log_types import PantherLogType


class StandardSlackAccessLogs(PantherDataModel):
    DataModelID: str = "Standard.Slack.AccessLogs"
    DisplayName: str = "Slack Access Logs"
    Enabled: bool = True
    LogTypes: List[str] = [PantherLogType.Slack_AccessLogs]
    Mappings: List[PantherDataModelMapping] = [
        PantherDataModelMapping(Name="source_ip", Path="ip"),
        PantherDataModelMapping(Name="user_agent", Path="user_agent"),
        PantherDataModelMapping(Name="actor_user", Path="username"),
    ]



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.log_types import PantherLogType

box_large_number_permission_updates_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Regular Event",
        ExpectedResult=False,
        Log={
            "type": "event",
            "additional_details": '{"key": "value"}',
            "created_by": {
                "id": "12345678",
                "type": "user",
                "login": "cat@example",
                "name": "Bob Cat",
            },
            "event_type": "DELETE",
        },
    ),
    PantherRuleTest(
        Name="User Permission Change",
        ExpectedResult=True,
        Log={
            "type": "event",
            "additional_details": '{"key": "value"}',
            "created_by": {
                "id": "12345678",
                "type": "user",
                "login": "cat@example",
                "name": "Bob Cat",
            },
            "event_type": "CHANGE_FOLDER_PERMISSION",
            "source": {
                "id": "12345678",
                "type": "user",
                "login": "user@example",
                "name": "Bob Cat",
            },
        },
    ),
    PantherRuleTest(
        Name="User Shares Item",
        ExpectedResult=True,
        Log={
            "type": "event",
            "additional_details": '{"key": "value"}',
            "created_by": {
                "id": "12345678",
                "type": "user",
                "login": "cat@example",
                "name": "Bob Cat",
            },
            "event_type": "ITEM_SHARED_CREATE",
            "source": {
                "id": "12345678",
                "type": "user",
                "login": "user@example",
                "name": "Bob Cat",
            },
        },
    ),
]


class BoxLargeNumberPermissionUpdates(PantherRule):
    RuleID = "Box.Large.Number.Permission.Updates-prototype"
    DisplayName = "Box Large Number of Permission Changes"
    LogTypes = [PantherLogType.Box_Event]
    Tags = ["Box", "Privilege Escalation:Abuse Elevation Control Mechanism"]
    Reports = {"MITRE ATT&CK": ["TA0004:T1548"]}
    Severity = PantherSeverity.Low
    Description = "A user has exceeded the threshold for number of folder permission changes within a single time frame.\n"
    Reference = (
        "https://support.box.com/hc/en-us/articles/360043697254-Understanding-Folder-Permissions"
    )
    Runbook = "Investigate whether this user's activity is expected.\n"
    SummaryAttributes = ["ip_address"]
    Threshold = 100
    Tests = box_large_number_permission_updates_tests
    PERMISSION_UPDATE_EVENT_TYPES = {
        "CHANGE_FOLDER_PERMISSION",
        "ITEM_SHARED_CREATE",
        "ITEM_SHARED",
        "SHARE",
    }

    def rule(self, event):
        return event.get("event_type") in self.PERMISSION_UPDATE_EVENT_TYPES

    def title(self, event):
        return f"User [{deep_get(event, 'created_by', 'login', default='<UNKNOWN_USER>')}] exceeded threshold for number of permission changes in the configured time frame."



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.log_types import PantherLogType

one_login_user_assumption_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="User assumed their own account",
        ExpectedResult=False,
        Log={
            "event_type_id": "240",
            "actor_user_id": 123456,
            "actor_user_name": "Bob Cat",
            "user_id": 123456,
            "user_name": "Bob Cat",
        },
    ),
    PantherRuleTest(
        Name="User assumed another user's account",
        ExpectedResult=True,
        Log={
            "event_type_id": "3",
            "actor_user_id": 654321,
            "actor_user_name": "Mountain Lion",
            "user_id": 123456,
            "user_name": "Bob Cat",
        },
    ),
]


class OneLoginUserAssumption(PantherRule):
    RuleID = "OneLogin.UserAssumption-prototype"
    DisplayName = "OneLogin User Assumed Another User"
    LogTypes = [PantherLogType.OneLogin_Events]
    Tags = ["OneLogin", "Lateral Movement:Use Alternate Authentication Material"]
    Reports = {"MITRE ATT&CK": ["TA0008:T1550"]}
    Severity = PantherSeverity.Low
    Description = "User assumed another user account"
    Reference = "https://onelogin.service-now.com/kb_view_customer.do?sysparm_article=KB0010594#:~:text=Prerequisites,Actions%20and%20select%20Assume%20User."
    Runbook = "Investigate whether this was authorized access.\n"
    SummaryAttributes = ["account_id", "user_name", "user_id"]
    Tests = one_login_user_assumption_tests

    def rule(self, event):
        # check that this is a user assumption event; event id 3
        return str(event.get("event_type_id")) == "3" and event.get(
            "actor_user_id", "UNKNOWN_USER"
        ) != event.get("user_id", "UNKNOWN_USER")

    def title(self, event):
        return f"A user [{event.get('actor_user_name', '<UNKNOWN_USER>')}] assumed another user [{event.get('user_name', '<UNKNOWN_USER>')}] account"



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.log_types import PantherLogType

asana_workspace_guest_invite_permissions_anyone_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Anyone Allowed Guest Invite",
        ExpectedResult=True,
        Log={
            "actor": {
                "actor_type": "user",
                "email": "homer.simpson@example.io",
                "gid": "12345",
                "name": "Homer Simpson",
            },
            "context": {
                "client_ip_address": "12.12.12.12",
                "context_type": "web",
                "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
            },
            "created_at": "2022-12-16 19:30:26.15",
            "details": {"new_value": "anyone", "old_value": "admins_only"},
            "event_category": "admin_settings",
            "event_type": "workspace_guest_invite_permissions_changed",
            "gid": "12345",
            "resource": {"gid": "12345", "name": "Example IO", "resource_type": "workspace"},
        },
    ),
    PantherRuleTest(
        Name="Other",
        ExpectedResult=False,
        Log={
            "actor": {
                "actor_type": "user",
                "email": "homer.simpson@simpsons.com",
                "gid": "1234567890",
                "name": "Homer Simpson",
            },
            "context": {
                "client_ip_address": "1.2.3.4",
                "context_type": "web",
                "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
            },
            "created_at": "2022-12-16 19:32:00.922",
            "details": {},
            "event_category": "admin_settings",
            "event_type": "workspace_form_link_authentication_required_disabled",
            "gid": "1234567890",
            "resource": {"gid": "111234", "name": "Simpsons Lab", "resource_type": "workspace"},
        },
    ),
]


class AsanaWorkspaceGuestInvitePermissionsAnyone(PantherRule):
    Description = "Typically inviting guests to Asana is permitted by few users. Enabling anyone to invite guests can potentially lead to unauthorized users gaining access to Asana."
    DisplayName = "Asana Workspace Guest Invite Permissions Anyone"
    Reference = "https://help.asana.com/hc/en-us/articles/14109494654875-Admin-console#:~:text=Google%20SSO%20password.-,Guest%20invite%20controls,-Super%20admins%20of"
    Severity = PantherSeverity.Low
    LogTypes = [PantherLogType.Asana_Audit]
    RuleID = "Asana.Workspace.Guest.Invite.Permissions.Anyone-prototype"
    Tests = asana_workspace_guest_invite_permissions_anyone_tests

    def rule(self, event):
        return (
            event.get("event_type") == "workspace_guest_invite_permissions_changed"
            and deep_get(event, "details", "new_value") == "anyone"
        )

    def title(self, event):
        workspace = deep_get(event, "resource", "name", default="<WORKSPACE_NOT_FOUND>")
        actor = deep_get(event, "actor", "email", default="<ACTOR_NOT_FOUND>")
        return f"Asana Workspace [{workspace}] guest invite permissions changed to anyone by [{actor}]."



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.log_types import PantherLogType

g_suite_group_banned_user_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="User Added",
        ExpectedResult=False,
        Log={
            "id": {"applicationName": "groups_enterprise"},
            "actor": {"email": "homer.simpson@example.com"},
            "type": "moderator_action",
            "name": "add_member",
        },
    ),
    PantherRuleTest(
        Name="User Banned from Group",
        ExpectedResult=True,
        Log={
            "id": {"applicationName": "groups_enterprise"},
            "actor": {"email": "homer.simpson@example.com"},
            "type": "moderator_action",
            "name": "ban_user_with_moderation",
        },
    ),
]


class GSuiteGroupBannedUser(PantherRule):
    RuleID = "GSuite.GroupBannedUser-prototype"
    DisplayName = "GSuite User Banned from Group"
    LogTypes = [PantherLogType.GSuite_ActivityEvent]
    Tags = ["GSuite"]
    Severity = PantherSeverity.Low
    Description = "A GSuite user was banned from an enterprise group by moderator action.\n"
    Reference = "https://support.google.com/a/users/answer/9303224?hl=en&sjid=864417124752637253-EU"
    Runbook = (
        "Investigate the banned user to see if further disciplinary action needs to be taken.\n"
    )
    SummaryAttributes = ["actor:email"]
    Tests = g_suite_group_banned_user_tests

    def rule(self, event):
        if deep_get(event, "id", "applicationName") != "groups_enterprise":
            return False
        if event.get("type") == "moderator_action":
            return bool(event.get("name") == "ban_user_with_moderation")
        return False

    def title(self, event):
        return f"User [{deep_get(event, 'actor', 'email', default='<UNKNOWN_EMAIL>')}] banned another user from a group."



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.log_types import PantherLogType

duo_user_bypass_code_used_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="bypass_code_used",
        ExpectedResult=True,
        Log={
            "access_device": {"ip": "12.12.112.25", "os": "Mac OS X"},
            "auth_device": {"ip": "12.12.12.12"},
            "application": {"key": "D12345", "name": "Slack"},
            "event_type": "authentication",
            "factor": "duo_push",
            "reason": "bypass_user",
            "result": "success",
            "user": {"name": "example@example.io"},
        },
    ),
    PantherRuleTest(
        Name="good_auth",
        ExpectedResult=False,
        Log={
            "access_device": {"ip": "12.12.112.25", "os": "Mac OS X"},
            "auth_device": {"ip": "12.12.12.12"},
            "application": {"key": "D12345", "name": "Slack"},
            "event_type": "authentication",
            "factor": "duo_push",
            "reason": "user_approved",
            "result": "success",
            "user": {"name": "example@example.io"},
        },
    ),
    PantherRuleTest(
        Name="denied_old_creds",
        ExpectedResult=False,
        Log={
            "access_device": {"ip": "12.12.112.25", "os": "Mac OS X"},
            "auth_device": {"ip": "12.12.12.12"},
            "application": {"key": "D12345", "name": "Slack"},
            "event_type": "authentication",
            "factor": "duo_push",
            "reason": "out_of_date",
            "result": "denied",
            "user": {"name": "example@example.io"},
        },
    ),
]


class DUOUserBypassCodeUsed(PantherRule):
    RuleID = "DUO.User.BypassCode.Used-prototype"
    DisplayName = "Duo User Bypass Code Used"
    DedupPeriodMinutes = 5
    LogTypes = [PantherLogType.Duo_Authentication]
    Tags = ["Duo"]
    Severity = PantherSeverity.Low
    Description = "A Duo user's bypass code was used to authenticate"
    Reference = "https://duo.com/docs/adminapi#authentication-logs"
    Runbook = "Follow up with the user to confirm they used the bypass code themselves."
    Tests = duo_user_bypass_code_used_tests

    def rule(self, event):
        return event.get("reason") == "bypass_user" and event.get("result") == "success"

    def title(self, event):
        user = deep_get(event, "user", "name", default="Unknown")
        return f"Bypass code for Duo User [{user}] used"

    def alert_context(self, event):
        return {
            "factor": event.get("factor"),
            "reason": event.get("reason"),
            "user": deep_get(event, "user", "name", default=""),
            "os": deep_get(event, "access_device", "os", default=""),
            "ip_access": deep_get(event, "access_device", "ip", default=""),
            "ip_auth": deep_get(event, "auth_device", "ip", default=""),
            "application": deep_get(event, "application", "name", default=""),
        }



from typing import List

from pypanther.base import PantherDataModel, PantherDataModelMapping
from pypanther.log_types import PantherLogType


class StandardSlackIntegrationLogs(PantherDataModel):
    DataModelID: str = "Standard.Slack.IntegrationLogs"
    DisplayName: str = "Slack Integration Logs"
    Enabled: bool = True
    LogTypes: List[str] = [PantherLogType.Slack_IntegrationLogs]
    Mappings: List[PantherDataModelMapping] = [
        PantherDataModelMapping(Name="actor_user", Path="user_name")
    ]



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.helpers.panther_notion_helpers import notion_alert_context
from pypanther.log_types import PantherLogType

notion_workspace_public_page_added_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Public page added",
        ExpectedResult=True,
        Log={
            "event": {
                "id": "eeeeeeee-dddd-4444-bbbb-4444444444444",
                "timestamp": "2023-05-15T19:14:21.031Z",
                "workspace_id": "vvvvvvvv-dddd-4444-bbbb-666666666666",
                "actor": {
                    "object": "user",
                    "id": "44444444-cccc-7777-aaaa-666666666666",
                    "type": "person",
                    "person": {"email": "example@personemail.com"},
                },
                "ip_address": "00.000.00.000",
                "platform": "web",
                "type": "workspace.settings.public_homepage_added",
                "workspace.settings.public_homepage_added": {
                    "new_public_page": {
                        "type": "database_id",
                        "database_id": "4b801dc7-d724-4fbb-afd0-9885cbc12405",
                    }
                },
            }
        },
    ),
    PantherRuleTest(
        Name="Workspace Exported",
        ExpectedResult=False,
        Log={
            "event": {
                "actor": {
                    "id": "bd37477c-869d-418b-abdb-0fc727b38b5e",
                    "object": "user",
                    "person": {"email": "homer.simpson@yourcompany.io"},
                    "type": "person",
                },
                "details": {
                    "parent": {
                        "type": "workspace_id",
                        "workspace_id": "ab99as87-6abc-4dcf-808b-111999882299",
                    },
                    "target": {
                        "page_id": "3cd2c560-d1b9-474e-b46e-gh8899002763",
                        "type": "page_id",
                    },
                },
                "id": "d4b9963f-12a8-4b01-b597-233a140abf5e",
                "ip_address": "12.12.12.12",
                "platform": "web",
                "timestamp": "2023-06-01 18:57:07.486000000",
                "type": "page.exported",
                "workspace_id": "ea65b016-6abc-4dcf-808b-e119617b55d1",
            }
        },
    ),
]


class NotionWorkspacePublicPageAdded(PantherRule):
    RuleID = "Notion.Workspace.Public.Page.Added-prototype"
    DisplayName = "Notion Workspace public page added"
    LogTypes = [PantherLogType.Notion_AuditLogs]
    Tags = ["Notion", "Data Security", "Information Disclosure"]
    Severity = PantherSeverity.Info
    Description = "A Notion page was set to public in your worksace."
    Runbook = "A Notion page was made public. Check with the author to determine why this page was made public."
    Reference = "https://www.notion.so/help/public-pages-and-web-publishing"
    Tests = notion_workspace_public_page_added_tests

    def rule(self, event):
        event_type = event.deep_get("event", "type", default="<NO_EVENT_TYPE_FOUND>")
        return event_type == "workspace.settings.public_homepage_added"

    def title(self, event):
        actor = event.deep_get("event", "actor", "person", "email", default="<NO_EMAIL_FOUND>")
        wkspc_id = event.deep_get("event", "workspace_id", default="<NO_WORKSPACE_ID_FOUND>")
        db_id = deep_get(
            event,
            "event",
            "workspace.settings.public_homepage_added",
            "new_public_page",
            "database_id",
            default="<NO_DATABASE_ID_FOUND>",
        )
        return (
            f"Notion User [{actor}] added a new public homepage [{db_id}] in workspace [{wkspc_id}]"
        )

    def alert_context(self, event):
        context = notion_alert_context(event)
        workspace_id = event.deep_get("event", "workspace_id", default="<NO_WORKSPACE_ID_FOUND>")
        db_id = deep_get(
            event,
            "event",
            "workspace.settings.public_homepage_added",
            "new_public_page",
            "database_id",
            default="<NO_DATABASE_ID_FOUND>",
        )
        context["workspace_id"] = workspace_id
        context["page_id"] = db_id
        return context



from typing import List

import pypanther.helpers.panther_event_type_helpers as event_type
from pypanther.base import PantherDataModel, PantherDataModelMapping
from pypanther.log_types import PantherLogType

ADMIN_EVENTS = {
    "business.add_admin",
    "business.invite_admin",
    "team.promote_maintainer",
}

CONDITIONAL_ADMIN_EVENTS = {
    "team.add_repository",
}


def get_admin_role(event):
    action = event.get("action", "")
    permission = event.get("permission", "")
    if action in CONDITIONAL_ADMIN_EVENTS and permission == "admin":
        return action
    return action if action in ADMIN_EVENTS else "<UNKNOWN_ADMIN_ROLE>"


def get_event_type(event):
    action = event.get("action", "")
    permission = event.get("permission", "")
    if action in ADMIN_EVENTS:
        return event_type.ADMIN_ROLE_ASSIGNED
    if action in CONDITIONAL_ADMIN_EVENTS and permission == "admin":
        return event_type.ADMIN_ROLE_ASSIGNED
    if event.get("action", "") == "org.disable_two_factor_requirement":
        return event_type.MFA_DISABLED
    return None


class StandardGithubAudit(PantherDataModel):
    DataModelID: str = "Standard.Github.Audit"
    DisplayName: str = "Github Audit"
    Enabled: bool = True
    LogTypes: List[str] = [PantherLogType.GitHub_Audit]
    Mappings: List[PantherDataModelMapping] = [
        PantherDataModelMapping(Name="actor_user", Path="actor"),
        PantherDataModelMapping(Name="assigned_admin_role", Method=get_admin_role),
        PantherDataModelMapping(Name="event_type", Method=get_event_type),
        PantherDataModelMapping(Name="user", Path="user"),
    ]



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.log_types import PantherLogType

box_content_workflow_policy_violation_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Regular Event",
        ExpectedResult=False,
        Log={
            "type": "event",
            "additional_details": '{"key": "value"}',
            "created_by": {
                "id": "12345678",
                "type": "user",
                "login": "cat@example",
                "name": "Bob Cat",
            },
            "event_type": "DELETE",
        },
    ),
    PantherRuleTest(
        Name="Upload Policy Violation",
        ExpectedResult=True,
        Log={
            "type": "event",
            "additional_details": '{"key": "value"}',
            "created_by": {
                "id": "12345678",
                "type": "user",
                "login": "cat@example",
                "name": "Bob Cat",
            },
            "event_type": "CONTENT_WORKFLOW_UPLOAD_POLICY_VIOLATION",
            "source": {"id": "12345678", "type": "user", "login": "user@example"},
        },
    ),
    PantherRuleTest(
        Name="Sharing Policy Violation",
        ExpectedResult=True,
        Log={
            "type": "event",
            "additional_details": {"key": "value"},
            "created_by": {
                "id": "12345678",
                "type": "user",
                "login": "cat@example",
                "name": "Mountain Lion",
            },
            "event_type": "CONTENT_WORKFLOW_SHARING_POLICY_VIOLATION",
            "source": {"id": "12345678", "type": "user", "login": "user@example"},
        },
    ),
]


class BoxContentWorkflowPolicyViolation(PantherRule):
    RuleID = "Box.Content.Workflow.Policy.Violation-prototype"
    DisplayName = "Box Content Workflow Policy Violation"
    LogTypes = [PantherLogType.Box_Event]
    Tags = ["Box"]
    Severity = PantherSeverity.Low
    Description = "A user violated the content workflow policy.\n"
    Reference = "https://support.box.com/hc/en-us/articles/360043692594-Creating-a-Security-Policy"
    Runbook = "Investigate whether the user continues to violate the policy and take measure to ensure they understand policy.\n"
    SummaryAttributes = ["event_type"]
    Tests = box_content_workflow_policy_violation_tests
    POLICY_VIOLATIONS = {
        "CONTENT_WORKFLOW_UPLOAD_POLICY_VIOLATION",
        "CONTENT_WORKFLOW_SHARING_POLICY_VIOLATION",
    }

    def rule(self, event):
        return event.get("event_type") in self.POLICY_VIOLATIONS

    def title(self, event):
        return f"User [{deep_get(event, 'created_by', 'name', default='<UNKNOWN_USER>')}] violated a content workflow policy."



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import github_alert_context
from pypanther.log_types import PantherLogType

git_hub_org_moderators_add_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="GitHub - Org Moderator Added",
        ExpectedResult=True,
        Log={
            "_document_id": "Ab123",
            "action": "organization_moderators.add_user",
            "actor": "sarah78",
            "actor_location": {"country_code": "US"},
            "at_sign_timestamp": "2022-12-11 05:17:28.078",
            "created_at": "2022-12-11 05:17:28.078",
            "org": "example-io",
            "user": "john1987",
        },
    ),
    PantherRuleTest(
        Name="GitHub - Org Moderator removed",
        ExpectedResult=False,
        Log={
            "_document_id": "Ab123",
            "action": "organization_moderators.remove_user",
            "actor": "sarah78",
            "actor_location": {"country_code": "US"},
            "at_sign_timestamp": "2022-12-11 05:17:28.078",
            "created_at": "2022-12-11 05:17:28.078",
            "org": "example-io",
            "user": "john1987",
        },
    ),
]


class GitHubOrgModeratorsAdd(PantherRule):
    RuleID = "GitHub.Org.Moderators.Add-prototype"
    DisplayName = "GitHub User Added to Org Moderators"
    LogTypes = [PantherLogType.GitHub_Audit]
    Tags = ["GitHub", "Initial Access:Supply Chain Compromise"]
    Severity = PantherSeverity.Medium
    Description = "Detects when a user is added to a GitHub org's list of moderators."
    Reference = "https://docs.github.com/en/organizations/managing-peoples-access-to-your-organization-with-roles/managing-moderators-in-your-organization"
    Tests = git_hub_org_moderators_add_tests

    def rule(self, event):
        return event.get("action") == "organization_moderators.add_user"

    def title(self, event):
        return f"GitHub.Audit: User [{event.get('actor', '<UNKNOWN_ACTOR>')}] added user [{event.get('user', '<UNKNOWN_USER>')}] to moderators in [{event.get('org', '<UNKNOWN_ORG>')}]"

    def alert_context(self, event):
        return github_alert_context(event)



import shlex
from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.log_types import PantherLogType

osquery_linux_aws_command_executed_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="AWS command executed on MacOS",
        ExpectedResult=False,
        Log={
            "name": "pack_incident-response_shell_history",
            "action": "added",
            "decorations": {"platform": "darwin"},
            "columns": {
                "command": "aws sts get-caller-identity",
                "uid": "1000",
                "directory": "/home/ubuntu",
                "username": "ubuntu",
            },
        },
    ),
    PantherRuleTest(
        Name="AWS command executed",
        ExpectedResult=True,
        Log={
            "name": "pack_incident-response_shell_history",
            "action": "added",
            "columns": {
                "command": "aws s3 ls",
                "uid": "1000",
                "directory": "/home/ubuntu",
                "username": "ubuntu",
            },
        },
    ),
    PantherRuleTest(
        Name="Tail command executed",
        ExpectedResult=False,
        Log={
            "name": "pack_incident-response_shell_history",
            "action": "added",
            "columns": {
                "command": "tail -f /var/log/all",
                "uid": "1000",
                "directory": "/home/ubuntu",
                "username": "ubuntu",
            },
        },
    ),
    PantherRuleTest(
        Name="Command with quote executed",
        ExpectedResult=False,
        Log={
            "name": "pack_incident-response_shell_history",
            "action": "added",
            "columns": {
                "command": "git commit -m 'all done'",
                "uid": "1000",
                "directory": "/home/ubuntu",
                "username": "ubuntu",
            },
        },
    ),
    PantherRuleTest(
        Name="Invalid command ignored",
        ExpectedResult=False,
        Log={
            "name": "pack_incident-response_shell_history",
            "action": "added",
            "columns": {
                "command": "unopened '",
                "uid": "1000",
                "directory": "/home/ubuntu",
                "username": "ubuntu",
            },
        },
    ),
]


class OsqueryLinuxAWSCommandExecuted(PantherRule):
    RuleID = "Osquery.Linux.AWSCommandExecuted-prototype"
    DisplayName = "AWS command executed on the command line"
    LogTypes = [PantherLogType.Osquery_Differential]
    Tags = ["Osquery", "Linux", "Execution:User Execution"]
    Reports = {"MITRE ATT&CK": ["TA0002:T1204"]}
    Severity = PantherSeverity.Medium
    Description = "An AWS command was executed on a Linux instance"
    Runbook = "See which other commands were executed, and then remove IAM role causing the access"
    Reference = "https://attack.mitre.org/techniques/T1078/"
    SummaryAttributes = ["name", "action"]
    Tests = osquery_linux_aws_command_executed_tests
    PLATFORM_IGNORE_LIST = {"darwin"}

    def rule(self, event):
        # Filter out irrelevant logs & systems
        if (
            event.get("action") != "added"
            or "shell_history" not in event.get("name")
            or deep_get(event, "decorations", "platform") in self.PLATFORM_IGNORE_LIST
        ):
            return False
        command = deep_get(event, "columns", "command")
        if not command:
            return False
        try:
            command_args = shlex.split(command)
        except ValueError:
            # "No escaped character" or "No closing quotation", probably an invalid command
            return False
        if command_args[0] == "aws":
            return True
        return False

    def title(self, event):
        return f"User [{deep_get(event, 'columns', 'username', default='<UNKNOWN_USER>')}] issued an aws-cli command on [{event.get('hostIdentifier', '<UNKNOWN_HOST>')}]"



import abc
import contextlib
import copy
import inspect
import json
import logging
import os
import sys
from dataclasses import dataclass, field
from enum import Enum
from functools import total_ordering
from typing import Any, Callable, Dict, List, Literal, Mapping, Optional, Tuple, Type, Union
from unittest.mock import MagicMock, patch

from jsonpath_ng import Fields
from jsonpath_ng.ext import parse
from panther_core.data_model import DataModel
from panther_core.detection import DetectionResult
from panther_core.enriched_event import PantherEvent
from panther_core.exceptions import FunctionReturnTypeError, UnknownDestinationError
from panther_core.util import get_bool_env_var
from pydantic import BaseModel, NonNegativeInt, PositiveInt, TypeAdapter

from pypanther.log_types import PantherLogType
from pypanther.validate import NonEmptyUniqueList, UniqueList

logger = logging.getLogger(__name__)

# We want to default this to false as PAT will want detection output
DISABLE_OUTPUT = get_bool_env_var("DISABLE_DETECTION_OUTPUT", False)

TYPE_RULE = "RULE"
TYPE_SCHEDULED_RULE = "SCHEDULED_RULE"
TYPE_CORRELATION_RULE = "CORRELATION_RULE"

ERROR_TYPE_RULE = "RULE_ERROR"
ERROR_TYPE_SCHEDULED_RULE = "SCHEDULED_RULE_ERROR"
ERROR_TYPE_CORRELATION_RULE = "CORRELATION_RULE_ERROR"

# Maximum size for a dedup string
MAX_DEDUP_STRING_SIZE = 1000

# Maximum size for a generated field
MAX_GENERATED_FIELD_SIZE = 1000

# Maximum number of destinations
MAX_DESTINATIONS_SIZE = 10

# The limit for DDB is 400kb per item (we store this one in DDB) and
# the limit for SQS/SNS is 256KB. The limit of 200kb is an approximation - the other
# fields included in the request will be less than the remaining 56kb
MAX_ALERT_CONTEXT_SIZE = 200 * 1024  # 200kb

ALERT_CONTEXT_ERROR_KEY = "_error"

TRUNCATED_STRING_SUFFIX = "... (truncated)"

DEFAULT_DETECTION_DEDUP_PERIOD_MINS = 60

# Used to check dynamic severity output
SEVERITY_TYPES = ["INFO", "LOW", "MEDIUM", "HIGH", "CRITICAL"]
SEVERITY_DEFAULT = "DEFAULT"

RULE_METHOD = "rule"

ALERT_CONTEXT_METHOD = "alert_context"
DEDUP_METHOD = "dedup"
DESCRIPTION_METHOD = "description"
DESTINATIONS_METHOD = "destinations"
REFERENCE_METHOD = "reference"
RUNBOOK_METHOD = "runbook"
SEVERITY_METHOD = "severity"
TITLE_METHOD = "title"

# Auxiliary METHODS are optional
AUXILIARY_METHODS = (
    ALERT_CONTEXT_METHOD,
    DEDUP_METHOD,
    DESCRIPTION_METHOD,
    DESTINATIONS_METHOD,
    REFERENCE_METHOD,
    RUNBOOK_METHOD,
    SEVERITY_METHOD,
    TITLE_METHOD,
)


PANTHER_RULE_ALL_METHODS = [
    RULE_METHOD,
    SEVERITY_METHOD,
    TITLE_METHOD,
    DEDUP_METHOD,
    DESTINATIONS_METHOD,
    RUNBOOK_METHOD,
    REFERENCE_METHOD,
    DESCRIPTION_METHOD,
    ALERT_CONTEXT_METHOD,
]


PANTHER_RULE_ALL_ATTRS = [
    "CreateAlert",
    "DedupPeriodMinutes",
    "Description",
    "DisplayName",
    "Enabled",
    "LogTypes",
    "OutputIds",
    "Reference",
    "Reports",
    "RuleID",
    "Runbook",
    "ScheduledQueries",
    "Severity",
    "SummaryAttributes",
    "Tags",
    "Tests",
    "Threshold",
]


PANTHER_RULE_TEST_ALL_ATTRS = [
    "Name",
    "ExpectedResult",
    "Log",
    "Mocks",
]

PANTHER_RULE_MOCK_ALL_ATTRS = [
    "ObjectName",
    "ReturnValue",
    "SideEffect",
]


def try_asdict(item: Any) -> Any:
    if hasattr(item, "asdict"):
        return item.asdict()
    if isinstance(item, list):
        return [try_asdict(v) for v in item]
    if isinstance(item, Enum):
        return item.value
    return item


@total_ordering
class PantherSeverity(str, Enum):
    Info = "Info"
    Low = "Low"
    Medium = "Medium"
    High = "High"
    Critical = "Critical"

    def __lt__(self, other):
        return PantherSeverity.as_int(self.value) < PantherSeverity.as_int(other.value)

    @staticmethod
    def as_int(value: "PantherSeverity") -> int:
        if value.upper() == PantherSeverity.Info.upper():
            return 0
        if value.upper() == PantherSeverity.Low.upper():
            return 1
        if value.upper() == PantherSeverity.Medium.upper():
            return 2
        if value.upper() == PantherSeverity.High.upper():
            return 3
        if value.upper() == PantherSeverity.Critical.upper():
            return 4
        raise ValueError(f"Unknown severity: {value}")

    def __str__(self) -> str:
        """Returns a string representation of the class' value."""
        return self.value


@dataclass
class PantherRuleMock:
    ObjectName: str
    ReturnValue: Any = None
    SideEffect: Any = None

    def asdict(self):
        """Returns a dictionary representation of the class."""
        return {key: try_asdict(getattr(self, key)) for key in PANTHER_RULE_MOCK_ALL_ATTRS}


class FileLocationMeta(type):
    def __call__(cls, *args, **kwargs):
        frame = inspect.currentframe().f_back
        file_path = frame.f_globals.get("__file__", None)
        line_number = frame.f_lineno
        module = frame.f_globals.get("__name__", None)
        instance = super().__call__(
            *args, **kwargs, _file_path=file_path, _line_no=line_number, _module=module
        )
        return instance


@dataclass
class PantherRuleTest(metaclass=FileLocationMeta):
    Name: str
    ExpectedResult: bool
    Log: Dict | str
    Mocks: List[PantherRuleMock] = field(default_factory=list)
    _file_path: str = ""
    _line_no: int = 0
    _module: str = ""

    def log_data(self):
        if isinstance(self.Log, str):
            return json.loads(self.Log)
        return self.Log

    def location(self) -> str:
        return f"{self._file_path}:{self._line_no}"

    def asdict(self):
        """Returns a dictionary representation of the class."""
        return {key: try_asdict(getattr(self, key)) for key in PANTHER_RULE_TEST_ALL_ATTRS}


@dataclass
class PantherRuleTestResult:
    """
    PantherRuleTestResult is the output returned from running a PantherRuleTest
    on a PantherRule.

    Attributes:
        Passed: If true, the PantherRuleTest passed. False, otherwise.
        DetectionResult: The result of the run() function on the given PantherEvent.
        Test: The test that was given and created this result.
        Rule: The PantherRule the PantherRuleTest was run on.
    """

    Passed: bool
    DetectionResult: DetectionResult
    Test: PantherRuleTest
    Rule: "PantherRule"


class PantherRuleModel(BaseModel):
    CreateAlert: bool
    DedupPeriodMinutes: NonNegativeInt
    Description: str
    DisplayName: str
    Enabled: bool
    LogTypes: NonEmptyUniqueList[str]
    OutputIds: UniqueList[str]
    Reference: str
    Reports: Dict[str, NonEmptyUniqueList[str]]
    RuleID: str
    Runbook: str
    ScheduledQueries: UniqueList[str]
    Severity: PantherSeverity
    SummaryAttributes: UniqueList[str]
    Tags: UniqueList[str]
    Tests: List[PantherRuleTest]
    Threshold: PositiveInt


PantherRuleAdapter = TypeAdapter(PantherRuleModel)


DEFAULT_CREATE_ALERT = True
DEFAULT_DEDUP_PERIOD_MINUTES = 60
DEFAULT_DESCRIPTION = ""
DEFAULT_DISPLAY_NAME = ""
DEFAULT_ENABLED = True
DEFAULT_OUTPUT_IDS: List[str] = []
DEFAULT_REFERENCE = ""
DEFAULT_REPORTS: Dict[str, List[str]] = {}
DEFAULT_RUNBOOK = ""
DEFAULT_SCHEDULED_QUERIES: List[str] = []
DEFAULT_SUMMARY_ATTRIBUTES: List[str] = []
DEFAULT_TAGS: List[str] = []
DEFAULT_TESTS: List[PantherRuleTest] = []
DEFAULT_THRESHOLD = 1


def truncate(s: str, max_size: int):
    if len(s) > max_size:
        # If generated field exceeds max size, truncate it
        num_characters_to_keep = max_size - len(TRUNCATED_STRING_SUFFIX)
        return s[:num_characters_to_keep] + TRUNCATED_STRING_SUFFIX
    return s


SeverityType = Union[PantherSeverity | Literal["DEFAULT"] | str]


class PantherRule(metaclass=abc.ABCMeta):
    """A Panther rule class. This class should be subclassed to create a new rule."""

    LogTypes: List[PantherLogType | str]
    RuleID: str
    Severity: PantherSeverity | str
    CreateAlert: bool = DEFAULT_CREATE_ALERT
    DedupPeriodMinutes: NonNegativeInt = DEFAULT_DEDUP_PERIOD_MINUTES
    Description: str = DEFAULT_DESCRIPTION
    DisplayName: str = DEFAULT_DISPLAY_NAME
    Enabled: bool = DEFAULT_ENABLED
    OutputIds: List[str] = DEFAULT_OUTPUT_IDS
    Reference: str = DEFAULT_REFERENCE
    Reports: Dict[str, List[str]] = DEFAULT_REPORTS
    Runbook: str = DEFAULT_RUNBOOK
    ScheduledQueries: List[str] = DEFAULT_SCHEDULED_QUERIES
    SummaryAttributes: List[str] = DEFAULT_SUMMARY_ATTRIBUTES
    Tags: List[str] = DEFAULT_TAGS
    Tests: List[PantherRuleTest] = DEFAULT_TESTS
    Threshold: PositiveInt = DEFAULT_THRESHOLD

    def _analysis_type(self) -> str:
        return TYPE_RULE

    @classmethod
    def is_panther_managed(cls) -> bool:
        return cls.__module__.startswith("pypanther.rules")

    @abc.abstractmethod
    def rule(self, event: PantherEvent) -> bool:
        raise NotImplementedError("You must implement the rule method in your rule class.")

    def severity(self, event: PantherEvent) -> SeverityType:
        return self.Severity

    def title(self, event: PantherEvent) -> str:
        return self.DisplayName if self.DisplayName else self.RuleID

    def dedup(self, event: PantherEvent) -> str:
        return self.title(event)

    def destinations(self, event: PantherEvent) -> List[str]:
        return self.OutputIds

    def runbook(self, event: PantherEvent) -> str:
        return self.Runbook

    def reference(self, event: PantherEvent) -> str:
        return self.Reference

    def description(self, event: PantherEvent) -> str:
        return self.Description

    def alert_context(self, event: PantherEvent) -> Dict:
        return {}

    def __init_subclass__(cls, **kwargs):
        """Creates a copy of all class attributes to avoid mod
        child.Tags.append("Foo")
        parent.Tags.append("Foo") # not inherited by children of parent
        """
        for attr in PANTHER_RULE_ALL_ATTRS:
            if attr not in cls.__dict__:
                try:
                    v = getattr(cls, attr)
                except AttributeError:
                    v = None

                if v is not None:
                    setattr(cls, attr, copy.deepcopy(v))
        super().__init_subclass__(**kwargs)

    @classmethod
    def asdict(cls):
        """Returns a dictionary representation of the class."""
        return {
            key: try_asdict(getattr(cls, key))
            for key in PANTHER_RULE_ALL_ATTRS
            if hasattr(cls, key)
        }

    @classmethod
    def validate(cls):
        PantherRuleAdapter.validate_python(cls.asdict())

        # instantiation confirms that abstract methods are implemented
        cls()

    @classmethod
    def override(
        cls,
        LogTypes: Optional[List[str]] = None,
        RuleID: Optional[str] = None,
        Severity: Optional[PantherSeverity] = None,
        CreateAlert: Optional[bool] = None,
        DedupPeriodMinutes: Optional[NonNegativeInt] = None,
        Description: Optional[str] = None,
        DisplayName: Optional[str] = None,
        Enabled: Optional[bool] = None,
        OutputIds: Optional[List[str]] = None,
        Reference: Optional[str] = None,
        Runbook: Optional[str] = None,
        Reports: Optional[Dict[str, List[str]]] = None,
        ScheduledQueries: Optional[List[str]] = None,
        SummaryAttributes: Optional[List[str]] = None,
        Tags: Optional[List[str]] = None,
        Tests: Optional[List[PantherRuleTest]] = None,
        Threshold: Optional[PositiveInt] = None,
    ):
        for key, val in locals().items():
            if key == "cls":
                continue

            if val is not None:
                setattr(cls, key, val)

    @classmethod
    def run_tests(
        cls,
        get_data_model: Callable[[str], Optional[DataModel]],
    ) -> list[PantherRuleTestResult]:
        """
        Runs all PantherRuleTests in this PantherRules' Test attribute over this
        PantherRule.

        Parameters:
            get_data_model: a helper function that will return a PantherDataModel given a log type.

        Returns:
            a list of PantherRuleTestResult objects.
        """
        cls.validate()
        rule = cls()

        return [rule.run_test(test, get_data_model) for test in rule.Tests]

    def run_test(
        self,
        test: PantherRuleTest,
        get_data_model: Callable[[str], Optional[DataModel]],
    ) -> PantherRuleTestResult:
        """
        Runs a unit test over this PantherRule.

        Parameters:
            test: the PantherRuleTest to run.
            get_data_model: a helper function that will return a PantherDataModel given a log type.

        Returns:
            a PantherRuleTestResult with the test result. If the Passed attribute is True,
            then this tests passed.
        """
        log = test.log_data()
        log_type = log.get("p_log_type", "default")

        event = PantherEvent(log, get_data_model(log_type))

        patches: List[Any] = []
        for each_mock in test.Mocks:
            kwargs = {each_mock.ObjectName: MagicMock(return_value=each_mock.ReturnValue)}
            p = patch.multiple(test._module, **kwargs)
            try:
                p.start()
            except AttributeError:
                p = patch.multiple(self, **kwargs)
                p.start()

        try:
            detection_result = self.run(event, {}, {}, False)

            if (
                detection_result.detection_exception is not None
                or detection_result.detection_output != test.ExpectedResult
            ):
                return PantherRuleTestResult(
                    Passed=False,
                    DetectionResult=detection_result,
                    Test=test,
                    Rule=self,
                )

            aux_func_exceptions = {
                "title": detection_result.title_exception,
                "description": detection_result.description_exception,
                "reference": detection_result.reference_exception,
                "severity": detection_result.severity_exception,
                "runbook": detection_result.runbook_exception,
                "destinations": detection_result.destinations_exception,
                "dedup": detection_result.dedup_exception,
                "alert_context": detection_result.alert_context_exception,
            }

            if any(True for _, exc in aux_func_exceptions.items() if exc is not None):
                return PantherRuleTestResult(
                    Passed=False,
                    DetectionResult=detection_result,
                    Test=test,
                    Rule=self,
                )

        finally:
            for p in patches:
                p.stop()

        return PantherRuleTestResult(
            Passed=True,
            DetectionResult=detection_result,
            Test=test,
            Rule=self,
        )

    def run(
        self,
        event: PantherEvent,
        outputs: Dict,
        outputs_names: Dict,
        batch_mode: bool = True,
    ) -> DetectionResult:
        result = DetectionResult(
            detection_id=self.RuleID,
            detection_severity=self.Severity,
            detection_type=TYPE_RULE,
            # set default to not alert
            trigger_alert=False,
        )

        try:
            result.detection_output = self.rule(event)
            self._require_bool(self.rule.__name__, result.detection_output)
        except Exception as e:
            result.detection_exception = e

        if isinstance(result.detection_output, bool) and result.detection_output:
            result.trigger_alert = True
        if batch_mode and not result.trigger_alert:
            # In batch mode (log analysis), there is no need to run the rest of the functions
            # if the detection isn't going to trigger an alert
            return result

        self.ctx_mgr = noop
        if DISABLE_OUTPUT:
            self.ctx_mgr = suppress_output

        result.title_output, result.title_exception = self._get_title(event)
        result.description_output, result.description_exception = self._get_description(event)
        result.reference_output, result.reference_exception = self._get_reference(event)
        result.severity_output, result.severity_exception = self._get_severity(event)
        result.runbook_output, result.runbook_exception = self._get_runbook(event)
        result.destinations_output, result.destinations_exception = self._get_destinations(
            event,
            outputs,
            outputs_names,
        )
        result.dedup_output, result.dedup_exception = self._get_dedup(event)
        result.alert_context_output, result.alert_context_exception = self._get_alert_context(event)

        if batch_mode:
            # batch mode ignores errors
            # in the panther backend, we check if any error occured during running and if we get one,
            # we return a detection error instead of an alert. To make sure alerts are still returned,
            # we need to set these to None.
            result.title_exception = None
            result.description_exception = None
            result.reference_exception = None
            result.severity_exception = None
            result.runbook_exception = None
            result.destinations_exception = None
            result.dedup_exception = None
            result.alert_context_exception = None

        return result

    def _get_title(self, event: Mapping) -> Tuple[str, Optional[Exception]]:
        try:
            with self.ctx_mgr():
                title = self.title(event)

            self._require_str(self.title.__name__, title)
        except Exception as e:
            title = self.DisplayName
            if not title or not isinstance(title, str):
                title = self.RuleID
            return title, e

        return truncate(title, MAX_GENERATED_FIELD_SIZE), None

    # Returns the dedup string for this detection match
    def _get_dedup(self, event: Mapping) -> Tuple[Optional[str], Optional[Exception]]:
        e = None
        dedup_string = ""
        try:
            with self.ctx_mgr():
                dedup_string = self.dedup(event)

            self._require_str(self.dedup.__name__, dedup_string)
        except Exception as err:
            e = err

        if dedup_string == "" or not isinstance(dedup_string, str):
            dedup_string, _ = self._get_title(event)
            if dedup_string == "" or not isinstance(dedup_string, str):
                dedup_string = f"defaultDedupString:{self.RuleID}"

        return truncate(dedup_string, MAX_DEDUP_STRING_SIZE), e

    def _get_description(
        self,
        event: Mapping,
    ) -> Tuple[str, Optional[Exception]]:
        try:
            with self.ctx_mgr():
                description = self.description(event)

            self._require_str(self.description.__name__, description)
        except Exception as e:
            return "", e

        return truncate(description, MAX_GENERATED_FIELD_SIZE), None

    def _get_reference(self, event: Mapping) -> Tuple[str, Optional[Exception]]:
        try:
            with self.ctx_mgr():
                reference = self.reference(event)

            self._require_str(self.reference.__name__, reference)
        except Exception as e:
            return "", e

        return truncate(reference, MAX_GENERATED_FIELD_SIZE), None

    def _get_runbook(self, event: Mapping) -> Tuple[Optional[str], Optional[Exception]]:
        try:
            with self.ctx_mgr():
                runbook = self.runbook(event)

            self._require_str(self.runbook.__name__, runbook)
        except Exception as e:
            return "", e

        return truncate(runbook, MAX_GENERATED_FIELD_SIZE), None

    def _get_severity(self, event: Mapping) -> Tuple[Optional[str], Optional[Exception]]:
        try:
            with self.ctx_mgr():
                severity: str = self.severity(event)

            self._require_str(self.severity.__name__, severity)
            severity = severity.upper()
            if severity == SEVERITY_DEFAULT:
                return self.Severity, None
            if severity not in SEVERITY_TYPES:
                raise AssertionError(
                    f"Expected severity to be any of the following: [{str(SEVERITY_TYPES)}], got [{severity}] instead."
                )
        except Exception as e:
            return self.Severity, e

        return severity, None

    def _get_alert_context(self, event: Mapping) -> Tuple[Optional[str], Optional[Exception]]:
        try:
            with self.ctx_mgr():
                alert_context = self.alert_context(event)

            self._require_mapping(self.alert_context.__name__, alert_context)
            serialized_alert_context = json.dumps(alert_context, default=PantherEvent.json_encoder)
        except Exception as err:
            return json.dumps({ALERT_CONTEXT_ERROR_KEY: repr(err)}), err

        if len(serialized_alert_context) > MAX_ALERT_CONTEXT_SIZE:
            # If context exceeds max size, return empty one
            alert_context_error = (
                f"alert_context size is [{len(serialized_alert_context)}] characters,"
                f" bigger than maximum of [{MAX_ALERT_CONTEXT_SIZE}] characters"
            )
            return json.dumps({ALERT_CONTEXT_ERROR_KEY: alert_context_error}), None

        return serialized_alert_context, None

    def _get_destinations(
        self,
        event: Mapping,
        outputs: Dict,
        outputs_display_names: Dict,
    ) -> Tuple[Optional[List[str]], Optional[Exception]]:
        try:
            with self.ctx_mgr():
                destinations = self.destinations(event)
            self._require_str_list(self.destinations.__name__, destinations)
        except Exception as e:
            return None, e

        # Return early if destinations returned None
        if destinations is None:
            return None, None

        # Return early if destinations is an empty list (alert dest. suppression)
        if len(destinations) == 0:
            return ["SKIP"], None

        # Check for (in)valid destinations
        invalid_destinations = []
        standardized_destinations: List[str] = []

        # Standardize the destinations
        for each_destination in destinations:
            # case for valid display name
            if (
                each_destination in outputs_display_names
                and outputs_display_names[each_destination].destination_id
                not in standardized_destinations
            ):
                standardized_destinations.append(
                    outputs_display_names[each_destination].destination_id
                )
            # case for valid UUIDv4
            elif each_destination in outputs and each_destination not in standardized_destinations:
                standardized_destinations.append(each_destination)
            else:
                invalid_destinations.append(each_destination)

        if invalid_destinations:
            try:
                # raise to get a stack trace
                raise UnknownDestinationError("Invalid Destinations", invalid_destinations)
            except UnknownDestinationError as e:
                return None, e

        if len(standardized_destinations) > MAX_DESTINATIONS_SIZE:
            # If generated field exceeds max size, truncate it
            standardized_destinations = standardized_destinations[:MAX_DESTINATIONS_SIZE]

        return standardized_destinations, None

    def _require_bool(self, method_name: str, value: Any):
        return self._require_scalar(method_name, bool, value)

    def _require_str(self, method_name: str, value: Any):
        return self._require_scalar(method_name, str, value)

    def _require_mapping(self, method_name: str, value: Any):
        return self._require_scalar(method_name, Mapping, value)

    def _require_scalar(self, method_name: str, typ: Type, value: Any):
        if not isinstance(value, typ):
            raise FunctionReturnTypeError(
                f"detection [{self.RuleID}] method [{method_name}] returned [{type(value).__name__}], expected [{typ.__name__}]"
            )

    def _require_str_list(self, method_name: str, value: Any):
        if value is None:
            return
        if not isinstance(value, list) or not all(isinstance(x, (str, bool)) for x in value):
            raise FunctionReturnTypeError(
                "detection [{}] method [{}] returned [{}], expected a list".format(
                    self.RuleID, method_name, type(value).__name__
                )
            )


@contextlib.contextmanager
def suppress_output():
    original_stdout = sys.stdout
    original_stderr = sys.stderr

    sys.stdout = open(os.devnull, "w")
    sys.stderr = open(os.devnull, "w")
    try:
        yield
    finally:
        sys.stdout = original_stdout
        sys.stderr = original_stderr


@contextlib.contextmanager
def noop():
    yield


@dataclass
class PantherDataModelMapping:
    Name: str
    Path: Optional[str] = None
    Method: Optional[Callable] = None


class PantherDataModel:
    DataModelID: str
    DisplayName: str
    Enabled: bool
    LogTypes: List[str]
    Mappings: List[PantherDataModelMapping]

    def __init__(self) -> None:
        self.paths: Dict[str, Fields] = {}
        self.methods: Dict[str, Callable] = {}

        for mapping in self.Mappings:
            if not mapping.Name:
                raise AssertionError(
                    f"DataModel [{self.DataModelID}] is missing required field: [Name]"
                )
            if mapping.Path:
                self.paths[mapping.Name] = parse(mapping.Path)
            elif mapping.Method:
                self.methods[mapping.Name] = mapping.Method
            else:
                raise AssertionError(
                    f"DataModel [{self.DataModelID}] must define one of: [Path, Method]"
                )



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.helpers.panther_tines_helpers import tines_alert_context
from pypanther.log_types import PantherLogType

tines_custom_certificate_authority_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Tines Login",
        ExpectedResult=False,
        Log={
            "created_at": "2023-05-17 14:45:19",
            "id": 7888888,
            "operation_name": "Login",
            "request_ip": "12.12.12.12",
            "request_user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
            "tenant_id": "8888",
            "user_email": "user@company.com",
            "user_id": "17171",
            "user_name": "user at company dot com",
        },
    ),
    PantherRuleTest(
        Name="Tines Custom CA set",
        ExpectedResult=True,
        Log={
            "created_at": "2023-05-18 22:54:11",
            "id": 7111111,
            "inputs": {},
            "operation_name": "CustomCertificateAuthoritySet",
            "request_ip": "12.12.12.12",
            "request_user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
            "tenant_id": "8888",
            "user_email": "user@company.com",
            "user_id": "17171",
            "user_name": "user at company dot com",
        },
    ),
]


class TinesCustomCertificateAuthority(PantherRule):
    RuleID = "Tines.Custom.CertificateAuthority-prototype"
    DisplayName = "Tines Custom CertificateAuthority setting changed"
    LogTypes = [PantherLogType.Tines_Audit]
    Tags = ["Tines", "IAM - Credential Security"]
    Reference = "https://www.tines.com/docs/admin/custom-certificate-authority"
    Severity = PantherSeverity.High
    Description = "Detects when Tines Custom CertificateAuthority settings are changed\n"
    SummaryAttributes = ["user_id", "operation_name", "tenant_id", "request_ip"]
    Tests = tines_custom_certificate_authority_tests
    ACTIONS = ["CustomCertificateAuthoritySet"]

    def rule(self, event):
        action = deep_get(event, "operation_name", default="<NO_OPERATION_NAME>")
        return action in self.ACTIONS

    def title(self, event):
        action = deep_get(event, "operation_name", default="<NO_OPERATION_NAME>")
        return f"Tines: [{action}] by [{deep_get(event, 'user_email', default='<NO_USEREMAIL>')}]"

    def alert_context(self, event):
        return tines_alert_context(event)

    def dedup(self, event):
        return f"{deep_get(event, 'user_id', default='<NO_USERID>')}_{deep_get(event, 'operation_name', default='<NO_OPERATION>')}"



import argparse
import importlib
import logging
import sys

from gql.transport.aiohttp import log as aiohttp_logger

from pypanther import testing, upload
from pypanther.custom_logging import setup_logging
from pypanther.vendor.panther_analysis_tool import util
from pypanther.vendor.panther_analysis_tool.command import standard_args
from pypanther.vendor.panther_analysis_tool.config import dynaconf_argparse_merge, setup_dynaconf


def run():
    setup_logging()

    parser = setup_parser()
    args = parser.parse_args()
    if args.command is None:
        parser.print_help()
        return

    if args.debug:
        logging.getLogger().setLevel(logging.DEBUG)
    else:
        aiohttp_logger.setLevel(logging.WARNING)

    config_file_settings = setup_dynaconf()
    dynaconf_argparse_merge(vars(args), config_file_settings)

    try:
        return_code, out = args.func(args)
    except util.BackendNotFoundException as err:
        logging.error('Backend not found: "%s"', err)
        return 1
    except Exception as err:  # pylint: disable=broad-except
        # Catch arbitrary exceptions without printing help message
        logging.warning('Unhandled exception: "%s"', err, exc_info=err, stack_info=True)
        logging.debug("Full error traceback:", exc_info=err)
        return 1

    if return_code > 0 and out:
        logging.error(out)
    elif return_code == 0 and out:
        logging.info(out)

    sys.exit(return_code)


def setup_parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(
        description="Command line tool for uploading files.", prog="pypanther"
    )
    parser.add_argument("--debug", action="store_true", dest="debug")
    subparsers = parser.add_subparsers(dest="command", help="Available commands")

    # Upload command
    upload_parser = subparsers.add_parser("upload", help="Upload a file")

    standard_args.for_public_api(upload_parser, required=False)
    upload_parser.set_defaults(func=util.func_with_backend(upload.run))
    upload_parser.add_argument(
        "--max-retries",
        help="Retry to upload on a failure for a maximum number of times",
        default=10,
        type=int,
        required=False,
    )
    upload_parser.add_argument(
        "--skip-tests",
        help="Skip running tests and go directly to upload",
        default=False,
        required=False,
        action="store_true",
    )

    # Test command
    test_parser = subparsers.add_parser("test", help="run tests")
    test_parser.set_defaults(func=testing.run)

    # Version command
    version_parser = subparsers.add_parser("version", help="version")
    version_parser.set_defaults(func=version)

    return parser


def version(args):
    print(importlib.metadata.version("pypanther"))
    return 0, ""


if __name__ == "__main__":
    run()



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.log_types import PantherLogType

osquery_ossec_rootkit_detected_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Rootkit Detected",
        ExpectedResult=True,
        Log={
            "action": "added",
            "calendarTime": "Tue Sep 11 16:14:21 2018 UTC",
            "columns": {
                "build_distro": "10.12",
                "build_platform": "darwin",
                "config_hash": "1111",
                "config_valid": "1",
                "counter": "14",
                "global_state": "0",
                "extensions": "active",
                "instance_id": "1111",
                "pid": "223",
                "resident_size": "54894592",
                "start_time": "1536634519",
                "system_time": "12472",
                "user_time": "31800",
                "uuid": "37821E12-CC8A-5AA3-A90C-FAB28A5BF8F9",
                "version": "3.2.6",
                "watcher": "92",
            },
            "counter": "255",
            "decorations": {"host_uuid": "1111", "environment": "corp"},
            "epoch": "0",
            "hostIdentifier": "test.lan",
            "log_type": "result",
            "name": "pack_ossec-rootkit_pwned",
            "unixTime": "1536682461",
        },
    ),
    PantherRuleTest(
        Name="Rootkit Not Detected",
        ExpectedResult=False,
        Log={
            "action": "added",
            "calendarTime": "Tue Sep 11 16:14:21 2018 UTC",
            "columns": {
                "build_distro": "10.12",
                "build_platform": "darwin",
                "config_hash": "1111",
                "config_valid": "1",
                "counter": "14",
                "global_state": "2",
                "extensions": "active",
                "instance_id": "1111",
                "pid": "223",
                "resident_size": "54894592",
                "start_time": "1536634519",
                "system_time": "12472",
                "user_time": "31800",
                "uuid": "37821E12-CC8A-5AA3-A90C-FAB28A5BF8F9",
                "version": "3.2.6",
                "watcher": "92",
            },
            "counter": "255",
            "decorations": {"host_uuid": "1111", "environment": "corp"},
            "epoch": "0",
            "hostIdentifier": "test.lan",
            "log_type": "result",
            "name": "pack_osquery-response_alf",
            "unixTime": "1536682461",
        },
    ),
]


class OsqueryOSSECRootkitDetected(PantherRule):
    RuleID = "Osquery.OSSECRootkitDetected-prototype"
    DisplayName = "OSSEC Rootkit Detected via Osquery"
    LogTypes = [PantherLogType.Osquery_Differential]
    Tags = ["Osquery", "Malware", "Defense Evasion:Rootkit"]
    Reports = {"MITRE ATT&CK": ["TA0005:T1014"]}
    Severity = PantherSeverity.Medium
    Description = "Checks if any results are returned for the Osquery OSSEC Rootkit pack.\n"
    Runbook = "Verify the presence of the rootkit and re-image the machine.\n"
    Reference = "https://panther.com/blog/osquery-log-analysis/"
    SummaryAttributes = ["name", "hostIdentifier", "action"]
    Tests = osquery_ossec_rootkit_detected_tests

    def rule(self, event):
        return "ossec-rootkit" in event.get("name", "") and event.get("action") == "added"

    def title(self, event):
        return f"OSSEC rootkit found on [{event.get('hostIdentifier')}]"



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import ZENDESK_CHANGE_DESCRIPTION
from pypanther.log_types import PantherLogType

zendesk_mobile_app_access_updated_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Zendesk - Mobile App Access Off",
        ExpectedResult=True,
        Log={
            "url": "https://myzendek.zendesk.com/api/v2/audit_logs/111222333444.json",
            "id": 123456789123,
            "action_label": "Updated",
            "actor_id": 123,
            "actor_name": "John Doe",
            "source_id": 123,
            "source_type": "account_setting",
            "source_label": "Zendesk Support Mobile App Access",
            "action": "create",
            "change_description": "Disabled",
            "ip_address": "127.0.0.1",
            "created_at": "2021-05-28T18:39:50Z",
            "p_log_type": "Zendesk.Audit",
        },
    ),
    PantherRuleTest(
        Name="Zendesk - Mobile App Access On",
        ExpectedResult=True,
        Log={
            "url": "https://myzendek.zendesk.com/api/v2/audit_logs/111222333444.json",
            "id": 123456789123,
            "action_label": "Updated",
            "actor_id": 123,
            "actor_name": "John Doe",
            "source_id": 123,
            "source_type": "account_setting",
            "source_label": "Zendesk Support Mobile App Access",
            "action": "create",
            "change_description": "Enabled",
            "ip_address": "127.0.0.1",
            "created_at": "2021-05-28T18:39:50Z",
            "p_log_type": "Zendesk.Audit",
        },
    ),
    PantherRuleTest(
        Name="Zendesk - Credit Card Redaction",
        ExpectedResult=False,
        Log={
            "url": "https://myzendek.zendesk.com/api/v2/audit_logs/111222333444.json",
            "id": 123456789123,
            "action_label": "Updated",
            "actor_id": 123,
            "actor_name": "John Doe",
            "source_id": 123,
            "source_type": "account_setting",
            "source_label": "Credit Card Redaction",
            "action": "create",
            "change_description": "Enabled",
            "ip_address": "127.0.0.1",
            "created_at": "2021-05-28T18:39:50Z",
            "p_log_type": "Zendesk.Audit",
        },
    ),
]


class ZendeskMobileAppAccessUpdated(PantherRule):
    RuleID = "Zendesk.MobileAppAccessUpdated-prototype"
    DisplayName = "Zendesk Mobile App Access Modified"
    LogTypes = [PantherLogType.Zendesk_Audit]
    Tags = ["Zendesk", "Persistence:Valid Accounts"]
    Reports = {"MITRE ATT&CK": ["TA0003:T1078"]}
    Severity = PantherSeverity.Medium
    Description = "A user updated account setting that enabled or disabled mobile app access."
    Reference = "https://support.zendesk.com/hc/en-us/articles/4408846407066-About-the-Zendesk-Support-mobile-app#:~:text=More%20settings.-,Configuring%20the%20mobile%20app,-Activate%20the%20new"
    SummaryAttributes = ["p_any_ip_addresses"]
    Tests = zendesk_mobile_app_access_updated_tests
    MOBILE_APP_ACTIONS = {"create", "update"}

    def rule(self, event):
        return (
            event.get("source_type") == "account_setting"
            and event.get("action", "") in self.MOBILE_APP_ACTIONS
            and (event.get("source_label", "") == "Zendesk Support Mobile App Access")
        )

    def title(self, event):
        action = event.get(ZENDESK_CHANGE_DESCRIPTION, "<UNKNOWN_ACTION>")
        return f"User [{event.udm('actor_user')}] {action} mobile app access"

    def severity(self, event):
        if event.get(ZENDESK_CHANGE_DESCRIPTION, "").lower() == "disabled":
            return "INFO"
        return "MEDIUM"



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import deep_get, slack_alert_context
from pypanther.log_types import PantherLogType

slack_audit_logs_app_removed_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="App Restricted",
        ExpectedResult=True,
        Log={
            "action": "app_restricted",
            "actor": {
                "type": "user",
                "user": {
                    "email": "user@example.com",
                    "id": "W012J3FEWAU",
                    "name": "primary-owner",
                    "team": "E012MH3HS94",
                },
            },
            "context": {
                "ip_address": "1.2.3.4",
                "location": {
                    "domain": "test-panther-1",
                    "id": "T01770N79GB",
                    "name": "test-workspace-1",
                    "type": "workspace",
                },
                "ua": "Go-http-client/2.0",
            },
            "date_create": "2021-06-08 22:16:15",
            "details": {"app_owner_id": "W012J3AEWAU", "is_internal_integration": True},
            "entity": {
                "app": {
                    "id": "A012F34BFEF",
                    "is_directory_approved": False,
                    "is_distributed": False,
                    "name": "app-name",
                    "scopes": ["admin"],
                },
                "type": "app",
            },
        },
    ),
    PantherRuleTest(
        Name="App Uninstalled",
        ExpectedResult=True,
        Log={
            "action": "app_uninstalled",
            "actor": {
                "type": "user",
                "user": {
                    "email": "user@example.com",
                    "id": "W012J3FEWAU",
                    "name": "primary-owner",
                    "team": "E012MH3HS94",
                },
            },
            "context": {
                "ip_address": "1.2.3.4",
                "location": {
                    "domain": "test-panther-1",
                    "id": "T01770N79GB",
                    "name": "test-workspace-1",
                    "type": "workspace",
                },
                "ua": "Go-http-client/2.0",
            },
            "date_create": "2021-06-08 22:16:15",
            "details": {"app_owner_id": "W012J3AEWAU", "is_internal_integration": True},
            "entity": {
                "app": {
                    "id": "A012F34BFEF",
                    "is_directory_approved": False,
                    "is_distributed": False,
                    "name": "app-name",
                    "scopes": ["admin"],
                },
                "type": "app",
            },
        },
    ),
    PantherRuleTest(
        Name="App removed from workspace",
        ExpectedResult=True,
        Log={
            "action": "org_app_workspace_removed",
            "actor": {
                "type": "user",
                "user": {
                    "email": "user@example.com",
                    "id": "W012J3FEWAU",
                    "name": "primary-owner",
                    "team": "E012MH3HS94",
                },
            },
            "context": {
                "ip_address": "1.2.3.4",
                "location": {
                    "domain": "test-panther-1",
                    "id": "T01770N79GB",
                    "name": "test-workspace-1",
                    "type": "workspace",
                },
                "ua": "Go-http-client/2.0",
            },
            "date_create": "2021-06-08 22:16:15",
            "details": {"app_owner_id": "W012J3AEWAU", "is_internal_integration": True},
            "entity": {
                "app": {
                    "id": "A012F34BFEF",
                    "is_directory_approved": False,
                    "is_distributed": False,
                    "name": "app-name",
                    "scopes": ["admin"],
                },
                "type": "app",
            },
        },
    ),
    PantherRuleTest(
        Name="User Logout",
        ExpectedResult=False,
        Log={
            "action": "user_logout",
            "actor": {
                "type": "user",
                "user": {
                    "email": "user@example.com",
                    "id": "W012J3FEWAU",
                    "name": "primary-owner",
                    "team": "T01234N56GB",
                },
            },
            "context": {
                "ip_address": "1.2.3.4",
                "location": {
                    "domain": "test-workspace-1",
                    "id": "T01234N56GB",
                    "name": "test-workspace-1",
                    "type": "workspace",
                },
                "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
            },
            "date_create": "2022-07-28 15:22:32",
            "entity": {
                "type": "user",
                "user": {
                    "email": "user@example.com",
                    "id": "W012J3FEWAU",
                    "name": "primary-owner",
                    "team": "T01234N56GB",
                },
            },
            "id": "72cac009-9eb3-4dde-bac6-ee49a32a1789",
        },
    ),
]


class SlackAuditLogsAppRemoved(PantherRule):
    RuleID = "Slack.AuditLogs.AppRemoved-prototype"
    DisplayName = "Slack App Removed"
    LogTypes = [PantherLogType.Slack_AuditLogs]
    Tags = [
        "Slack",
        "Impact",
        "Service Stop",
        "Defense Evasion",
        "Indicator Removal",
        "Clear Persistence",
    ]
    Reports = {"MITRE ATT&CK": ["TA0040:T1489", "TA0005:T1070.009"]}
    Severity = PantherSeverity.Medium
    Description = "Detects when a Slack App has been removed"
    Reference = "https://slack.com/intl/en-gb/help/articles/360003125231-Remove-apps-and-customised-integrations-from-your-workspace"
    SummaryAttributes = ["p_any_ip_addresses", "p_any_emails"]
    Tests = slack_audit_logs_app_removed_tests
    APP_REMOVED_ACTIONS = ["app_restricted", "app_uninstalled", "org_app_workspace_removed"]

    def rule(self, event):
        return event.get("action") in self.APP_REMOVED_ACTIONS

    def title(self, event):
        return f"Slack App [{deep_get(event, 'entity', 'app', 'name')}] Removed by [{deep_get(event, 'actor', 'user', 'name')}]"

    def alert_context(self, event):
        return slack_alert_context(event)



import argparse
import logging
import os
from typing import Optional, Tuple

from pypanther.base import PantherRuleTestResult
from pypanther.cache import DATA_MODEL_CACHE
from pypanther.import_main import NoMainModuleError, import_main
from pypanther.registry import registered_rules


def run(args: argparse.Namespace) -> Tuple[int, str]:
    try:
        import_main(os.getcwd(), "main")
    except NoMainModuleError:
        logging.error("No main.py found")
        return 1, ""

    failed_test_results: list[list[PantherRuleTestResult]] = []
    for rule in registered_rules():
        results = rule.run_tests(DATA_MODEL_CACHE.data_model_of_logtype)
        failures = [result for result in results if not result.Passed]
        if len(failures) > 0:
            failed_test_results.append(failures)

    print_failed_test_results(failed_test_results)

    if len(failed_test_results) > 0:
        return 1, "One or more rule tests are failing"

    return 0, "All tests passed"


def print_failed_test_results(failed_test_results: list[list[PantherRuleTestResult]]) -> None:
    if len(failed_test_results) == 0:
        return

    test_failure_separator: Optional[str] = None
    single_test_failure_separator: Optional[str] = None
    terminal_cols: Optional[int] = None
    try:
        terminal_cols = os.get_terminal_size().columns
        test_failure_separator = "=" * terminal_cols
        single_test_failure_separator = "-" * terminal_cols
    except OSError:
        pass

    if test_failure_separator:
        print(test_failure_separator)

    for failed_results in failed_test_results:
        if len(failed_results) == 0:
            continue

        if terminal_cols:
            side_count = int((terminal_cols - len(failed_results[0].Rule.RuleID)) / 2)
            print(f"{' '*side_count}{failed_results[0].Rule.RuleID}{' '*side_count}")

        for failed_result in failed_results:

            if single_test_failure_separator:
                print(single_test_failure_separator)

            if failed_result.DetectionResult.detection_exception is not None:
                log_rule_func_exception(failed_result)

            aux_func_exceptions = {
                "title": failed_result.DetectionResult.title_exception,
                "description": failed_result.DetectionResult.description_exception,
                "reference": failed_result.DetectionResult.reference_exception,
                "severity": failed_result.DetectionResult.severity_exception,
                "runbook": failed_result.DetectionResult.runbook_exception,
                "destinations": failed_result.DetectionResult.destinations_exception,
                "dedup": failed_result.DetectionResult.dedup_exception,
                "alert_context": failed_result.DetectionResult.alert_context_exception,
            }

            had_aux_exc = False
            for method_name, exc in aux_func_exceptions.items():
                if exc:
                    had_aux_exc = True
                    log_aux_func_exception(failed_result, method_name, exc)

            if had_aux_exc:
                log_aux_func_failure(failed_result, aux_func_exceptions)

            if (
                failed_result.DetectionResult.detection_exception is None
                and failed_result.DetectionResult.detection_output
                != failed_result.Test.ExpectedResult
            ):
                log_rule_test_failure(failed_result)

        if test_failure_separator:
            print(test_failure_separator)


def log_rule_func_exception(failed_result: PantherRuleTestResult) -> None:
    logging.error(
        "%s: Exception in test '%s' calling rule(): '%s': %s",
        failed_result.Rule.RuleID,
        failed_result.Test.Name,
        failed_result.DetectionResult.detection_exception,
        failed_result.Test.location(),
        exc_info=failed_result.DetectionResult.detection_exception,
    )


def log_aux_func_exception(
    failed_result: PantherRuleTestResult, method_name: str, exc: Exception
) -> None:
    logging.warning(
        "%s: Exception in test '%s' calling %s()",
        failed_result.Rule.RuleID,
        failed_result.Test.Name,
        method_name,
        exc_info=exc,
    )


def log_rule_test_failure(failed_result: PantherRuleTestResult) -> None:
    logging.error(
        "%s: test '%s' returned the wrong result, expected %s but got %s: %s",
        failed_result.Rule.RuleID,
        failed_result.Test.Name,
        failed_result.Test.ExpectedResult,
        failed_result.DetectionResult.detection_output,
        failed_result.Test.location(),
    )


def log_aux_func_failure(
    failed_result: PantherRuleTestResult, aux_func_exceptions: dict[str, Exception]
) -> None:
    exc_msgs = [f"{name}()" for name, exc in aux_func_exceptions.items() if exc is not None]
    exc_msg = ", ".join(exc_msgs[:-1]) if len(exc_msgs) > 1 else exc_msgs[0]
    last_exc_msg = f" and {exc_msgs[-1]}" if len(exc_msgs) > 1 else ""

    logging.error(
        "%s: test '%s': %s%s raised an exception, see log output for stacktrace",
        failed_result.Rule.RuleID,
        failed_result.Test.Name,
        exc_msg,
        last_exc_msg,
    )



from typing import List

from pypanther.base import PantherDataModel, PantherDataModelMapping
from pypanther.log_types import PantherLogType


class StandardAWSS3ServerAccess(PantherDataModel):
    DataModelID: str = "Standard.AWS.S3ServerAccess"
    DisplayName: str = "AWS S3 Server Access"
    Enabled: bool = True
    LogTypes: List[str] = [PantherLogType.AWS_S3ServerAccess]
    Mappings: List[PantherDataModelMapping] = [
        PantherDataModelMapping(Name="http_status", Path="httpstatus"),
        PantherDataModelMapping(Name="source_ip", Path="remoteip"),
        PantherDataModelMapping(Name="user_agent", Path="useragent"),
    ]



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.helpers.panther_snyk_helpers import snyk_alert_context
from pypanther.log_types import PantherLogType

snyk_misc_settings_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Snyk Feature Flags changed",
        ExpectedResult=True,
        Log={
            "created": "2023-04-11 23:32:14.173",
            "event": "group.feature_flags.edit",
            "groupId": "8fffffff-1555-4444-b000-b55555555555",
            "orgId": "21111111-a222-4eee-8ddd-a99999999999",
            "userId": "05555555-3333-4ddd-8ccc-755555555555",
        },
    ),
    PantherRuleTest(
        Name="Snyk User Invite Revoke",
        ExpectedResult=False,
        Log={
            "content": {},
            "created": "2023-04-11 23:32:13.248",
            "event": "org.user.invite.revoke",
            "groupId": "8fffffff-1555-4444-b000-b55555555555",
            "orgId": "21111111-a222-4eee-8ddd-a99999999999",
            "userId": "05555555-3333-4ddd-8ccc-755555555555",
        },
    ),
]


class SnykMiscSettings(PantherRule):
    RuleID = "Snyk.Misc.Settings-prototype"
    DisplayName = "Snyk Miscellaneous Settings"
    LogTypes = [PantherLogType.Snyk_GroupAudit, PantherLogType.Snyk_OrgAudit]
    Tags = ["Snyk"]
    Reference = "https://docs.snyk.io/snyk-admin/manage-settings"
    Severity = PantherSeverity.Low
    Description = "Detects when Snyk settings that lack a clear security impact are changed\n"
    SummaryAttributes = ["event"]
    Tests = snyk_misc_settings_tests
    ACTIONS = ["group.cloud_config.settings.edit", "group.feature_flags.edit"]

    def rule(self, event):
        action = deep_get(event, "event", default="<NO_EVENT>")
        return action in self.ACTIONS

    def title(self, event):
        group_or_org = "<GROUP_OR_ORG>"
        operation = "<NO_OPERATION>"
        action = deep_get(event, "event", default="<NO_EVENT>")
        if "." in action:
            group_or_org = action.split(".")[0].title()
            operation = ".".join(action.split(".")[1:]).title()
        return f"Snyk: [{group_or_org}] Setting [{operation}] performed by [{deep_get(event, 'userId', default='<NO_USERID>')}]"

    def alert_context(self, event):
        return snyk_alert_context(event)

    def dedup(self, event):
        return f"{deep_get(event, 'userId', default='<NO_USERID>')}{deep_get(event, 'orgId', default='<NO_ORGID>')}{deep_get(event, 'groupId', default='<NO_GROUPID>')}{deep_get(event, 'event', default='<NO_EVENT>')}"



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import aws_rule_context
from pypanther.log_types import PantherLogType

awsvpc_healthy_log_status_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Healthy Log Status",
        ExpectedResult=False,
        Log={"log-status": "OK", "p_log_type": "AWS.VPCFlow"},
    ),
    PantherRuleTest(
        Name="Unhealthy Log Status",
        ExpectedResult=True,
        Log={"log-status": "SKIPDATA", "p_log_type": "AWS.VPCFlow"},
    ),
    PantherRuleTest(
        Name="Healthy Log Status - OCSF",
        ExpectedResult=False,
        Log={"status_code": "OK", "p_log_type": "OCSF.NetworkActivity"},
    ),
    PantherRuleTest(
        Name="Unhealthy Log Status - OCSF",
        ExpectedResult=True,
        Log={"status_code": "SKIPDATA", "p_log_type": "OCSF.NetworkActivity"},
    ),
]


class AWSVPCHealthyLogStatus(PantherRule):
    RuleID = "AWS.VPC.HealthyLogStatus-prototype"
    DisplayName = "AWS VPC Healthy Log Status"
    LogTypes = [PantherLogType.AWS_VPCFlow, PantherLogType.OCSF_NetworkActivity]
    Tags = ["AWS", "DataModel", "Security Control"]
    Severity = PantherSeverity.Low
    Description = "Checks for the log status `SKIP-DATA`, which indicates that data was lost either to an internal server error or due to capacity constraints.\n"
    Reference = "https://docs.aws.amazon.com/vpc/latest/userguide/flow-logs.html#flow-log-records"
    Runbook = "Determine if the cause of the issue is capacity constraints, and consider adjusting VPC Flow Log configurations accordingly.\n"
    Tests = awsvpc_healthy_log_status_tests

    def rule(self, event):
        return event.udm("log_status") == "SKIPDATA"

    def alert_context(self, event):
        return aws_rule_context(event)



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_notion_helpers import notion_alert_context
from pypanther.log_types import PantherLogType

notion_workspace_exported_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Workspace Exported",
        ExpectedResult=True,
        Log={
            "event": {
                "id": "...",
                "timestamp": "2023-06-02T20:16:41.217Z",
                "workspace_id": "..",
                "actor": {
                    "id": "..",
                    "object": "user",
                    "type": "person",
                    "person": {"email": "homer.simpson@yourcompany.io"},
                },
                "ip_address": "...",
                "platform": "mac-desktop",
                "type": "workspace.content_exported",
                "workspace.content_exported": {},
            }
        },
    ),
    PantherRuleTest(
        Name="Other Event",
        ExpectedResult=False,
        Log={
            "event": {
                "actor": {
                    "id": "bd37477c-869d-418b-abdb-0fc727b38b5e",
                    "object": "user",
                    "person": {"email": "homer.simpson@yourcompany.io"},
                    "type": "person",
                },
                "details": {
                    "parent": {
                        "type": "workspace_id",
                        "workspace_id": "ab99as87-6abc-4dcf-808b-111999882299",
                    },
                    "target": {
                        "page_id": "3cd2c560-d1b9-474e-b46e-gh8899002763",
                        "type": "page_id",
                    },
                },
                "id": "d4b9963f-12a8-4b01-b597-233a140abf5e",
                "ip_address": "12.12.12.12",
                "platform": "web",
                "timestamp": "2023-06-01 18:57:07.486000000",
                "type": "page.exported",
                "workspace_id": "ea65b016-6abc-4dcf-808b-e119617b55d1",
            }
        },
    ),
]


class NotionWorkspaceExported(PantherRule):
    RuleID = "Notion.Workspace.Exported-prototype"
    DisplayName = "Notion Workspace Exported"
    LogTypes = [PantherLogType.Notion_AuditLogs]
    Tags = ["Notion", "Data Security", "Data Exfiltration"]
    Severity = PantherSeverity.High
    Description = "A Notion User exported an existing workspace."
    Runbook = "Possible Data Exfiltration. Follow up with the Notion User to determine if this was done for a valid business reason."
    Reference = "https://www.notion.so/help/workspace-settings#export-an-entire-workspace"
    Tests = notion_workspace_exported_tests

    def rule(self, event):
        event_type = event.deep_get("event", "type", default="<NO_EVENT_TYPE_FOUND>")
        return event_type == "workspace.content_exported"

    def title(self, event):
        user = event.deep_get("event", "actor", "person", "email", default="<NO_USER_FOUND>")
        workspace_id = event.deep_get("event", "workspace_id", default="<NO_WORKSPACE_ID_FOUND>")
        return f"Notion User [{user}] exported a workspace with workspace id [{workspace_id}]."

    def alert_context(self, event):
        return notion_alert_context(event)



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.log_types import PantherLogType

g_suite_google_access_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Normal Login Event",
        ExpectedResult=False,
        Log={"id": {"applicationName": "login"}, "type": "login"},
    ),
    PantherRuleTest(
        Name="Resource Accessed by Google",
        ExpectedResult=True,
        Log={"id": {"applicationName": "access_transparency"}, "type": "GSUITE_RESOURCE"},
    ),
]


class GSuiteGoogleAccess(PantherRule):
    RuleID = "GSuite.GoogleAccess-prototype"
    DisplayName = "Google Accessed a GSuite Resource"
    LogTypes = [PantherLogType.GSuite_ActivityEvent]
    Tags = ["GSuite"]
    Severity = PantherSeverity.Low
    Description = "Google accessed one of your GSuite resources directly, most likely in response to a support incident.\n"
    Reference = "https://support.google.com/a/answer/9230474?hl=en"
    Runbook = "Your GSuite Super Admin can visit the Access Transparency report in the GSuite Admin Dashboard to see more details about the access.\n"
    SummaryAttributes = ["actor:email"]
    Tests = g_suite_google_access_tests

    def rule(self, event):
        if deep_get(event, "id", "applicationName") != "access_transparency":
            return False
        return bool(event.get("type") == "GSUITE_RESOURCE")



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.log_types import PantherLogType

git_hub_org_ip_allowlist_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="GitHub - IP Allow list modified",
        ExpectedResult=True,
        Log={
            "actor": "cat",
            "action": "ip_allow_list_entry.create",
            "created_at": 1621305118553,
            "p_log_type": "GitHub.Audit",
            "org": "my-org",
        },
    ),
    PantherRuleTest(
        Name="GitHub - IP Allow list disabled",
        ExpectedResult=True,
        Log={
            "actor": "cat",
            "action": "ip_allow_list.disable",
            "created_at": 1621305118553,
            "org": "my-org",
            "p_log_type": "GitHub.Audit",
        },
    ),
    PantherRuleTest(
        Name="GitHub - Non IP Allow list action",
        ExpectedResult=False,
        Log={
            "actor": "cat",
            "action": "org.invite_user",
            "created_at": 1621305118553,
            "org": "my-org",
            "p_log_type": "GitHub.Audit",
        },
    ),
]


class GitHubOrgIpAllowlist(PantherRule):
    RuleID = "GitHub.Org.IpAllowlist-prototype"
    DisplayName = "GitHub Org IP Allow List modified"
    LogTypes = [PantherLogType.GitHub_Audit]
    Tags = ["GitHub", "Persistence:Account Manipulation"]
    Reports = {"MITRE ATT&CK": ["TA0003:T1098"]}
    Severity = PantherSeverity.Medium
    SummaryAttributes = ["actor", "action"]
    Description = "Detects changes to a GitHub Org IP Allow List"
    Runbook = "Verify that the change was authorized and appropriate."
    Reference = "https://docs.github.com/en/apps/maintaining-github-apps/managing-allowed-ip-addresses-for-a-github-app"
    Tests = git_hub_org_ip_allowlist_tests
    ALLOWLIST_ACTIONS = [
        "ip_allow_list.enable",
        "ip_allow_list.disable",
        "ip_allow_list.enable_for_installed_apps",
        "ip_allow_list.disable_for_installed_apps",
        "ip_allow_list_entry.create",
        "ip_allow_list_entry.update",
        "ip_allow_list_entry.destroy",
    ]

    def rule(self, event):
        return (
            event.get("action").startswith("ip_allow_list")
            and event.get("action") in self.ALLOWLIST_ACTIONS
        )

    def title(self, event):
        return f"GitHub Org IP Allow list modified by {event.get('actor')}."



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_duo_helpers import deserialize_administrator_log_event_description
from pypanther.log_types import PantherLogType

duo_admin_action_marked_fraudulent_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="marked_fraud",
        ExpectedResult=True,
        Log={
            "action": "admin_2fa_error",
            "description": '{"ip_address": "12.12.12.12", "email": "example@example.io", "factor": "push", "error": "Login request reported as fraudulent."}',
            "isotimestamp": "2022-12-14 20:11:53",
            "timestamp": "2022-12-14 20:11:53",
            "username": "John P. Admin",
        },
    ),
    PantherRuleTest(
        Name="different_admin_action",
        ExpectedResult=False,
        Log={
            "action": "admin_update",
            "description": "{}",
            "isotimestamp": "2022-12-14 20:11:53",
            "timestamp": "2022-12-14 20:11:53",
            "username": "John P. Admin",
        },
    ),
]


class DUOAdminActionMarkedFraudulent(PantherRule):
    RuleID = "DUO.Admin.Action.MarkedFraudulent-prototype"
    DisplayName = "Duo Admin Marked Push Fraudulent"
    DedupPeriodMinutes = 15
    LogTypes = [PantherLogType.Duo_Administrator]
    Tags = ["Duo"]
    Severity = PantherSeverity.Medium
    Description = "A Duo push was marked fraudulent by an admin."
    Reference = "https://duo.com/docs/adminapi#administrator-logs"
    Runbook = "Follow up with the administrator to determine reasoning for marking fraud."
    Tests = duo_admin_action_marked_fraudulent_tests

    def rule(self, event):
        event_description = deserialize_administrator_log_event_description(event)
        return (
            event.get("action") == "admin_2fa_error"
            and "fraudulent" in event_description.get("error", "").lower()
        )

    def title(self, event):
        event_description = deserialize_administrator_log_event_description(event)
        admin_username = event.get("username", "Unknown")
        user_email = event_description.get("email", "Unknown")
        return (
            f"Duo Admin [{admin_username}] denied due to an anomalous 2FA push for [{user_email}]"
        )

    def alert_context(self, event):
        event_description = deserialize_administrator_log_event_description(event)
        return {
            "reason": event_description.get("error", ""),
            "reporting_admin": event.get("username", ""),
            "user": event_description.get("email", ""),
            "ip_address": event_description.get("ip_address", ""),
        }



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.log_types import PantherLogType

asana_service_account_created_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="New domain created",
        ExpectedResult=False,
        Log={
            "actor": {
                "actor_type": "user",
                "email": "homer.simpson@example.io",
                "gid": "12345",
                "name": "Homer Simpson",
            },
            "context": {
                "client_ip_address": "12.12.12.12",
                "context_type": "web",
                "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
            },
            "created_at": "2022-12-16 19:30:26.15",
            "details": {"new_value": "test.com"},
            "event_category": "admin_settings",
            "event_type": "workspace_associated_email_domain_added",
            "gid": "12345",
            "resource": {"gid": "12345", "name": "Example IO", "resource_type": "workspace"},
        },
    ),
    PantherRuleTest(
        Name="Slack svc acct",
        ExpectedResult=True,
        Log={
            "actor": {
                "actor_type": "user",
                "email": "homer.simpson@panther.io",
                "gid": "12345",
                "name": "Homer Simpson",
            },
            "context": {
                "client_ip_address": "12.12.12.12",
                "context_type": "web",
                "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
            },
            "created_at": "2022-12-16 19:28:18.396",
            "details": {},
            "event_category": "apps",
            "event_type": "service_account_created",
            "gid": "12345",
            "resource": {"gid": "12345", "name": "Slack Service Account", "resource_type": "user"},
        },
    ),
    PantherRuleTest(
        Name="Datadog svc acct",
        ExpectedResult=True,
        Log={
            "actor": {
                "actor_type": "user",
                "email": "homer.simpson@panther.io",
                "gid": "12345",
                "name": "Homer Simpson",
            },
            "context": {
                "client_ip_address": "12.12.12.12",
                "context_type": "web",
                "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
            },
            "created_at": "2022-12-16 19:28:18.396",
            "details": {},
            "event_category": "apps",
            "event_type": "service_account_created",
            "gid": "12345",
            "resource": {
                "gid": "12345",
                "name": "Datadog Service Account",
                "resource_type": "user",
            },
        },
    ),
]


class AsanaServiceAccountCreated(PantherRule):
    Description = "An Asana service account was created by someone in your organization."
    DisplayName = "Asana Service Account Created"
    Runbook = "Confirm this user acted with valid business intent and determine whether this activity was authorized."
    Reference = "https://help.asana.com/hc/en-us/articles/14217496838427-Service-Accounts"
    Severity = PantherSeverity.Medium
    LogTypes = [PantherLogType.Asana_Audit]
    RuleID = "Asana.Service.Account.Created-prototype"
    Tests = asana_service_account_created_tests

    def rule(self, event):
        return event.get("event_type", "<NO_EVENT_TYPE_FOUND>") == "service_account_created"

    def title(self, event):
        actor_email = deep_get(event, "actor", "email", default="<ACTOR_NOT_FOUND>")
        svc_acct_name = deep_get(event, "resource", "name", default="<SVC_ACCT_NAME_NOT_FOUND>")
        return f"Asana user [{actor_email}] created a new service account [{svc_acct_name}]."



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.log_types import PantherLogType

g_suite_user_suspended_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Normal Login Event",
        ExpectedResult=False,
        Log={
            "id": {"applicationName": "login"},
            "kind": "admin#reports#activity",
            "type": "account_warning",
            "name": "login_success",
            "parameters": {"affected_email_address": "bobert@ext.runpanther.io"},
        },
    ),
    PantherRuleTest(
        Name="Account Warning Not For User Suspended",
        ExpectedResult=False,
        Log={
            "id": {"applicationName": "login"},
            "kind": "admin#reports#activity",
            "type": "account_warning",
            "name": "suspicious_login ",
            "parameters": {"affected_email_address": "bobert@ext.runpanther.io"},
        },
    ),
    PantherRuleTest(
        Name="Account Warning For Suspended User",
        ExpectedResult=True,
        Log={
            "id": {"applicationName": "login"},
            "kind": "admin#reports#activity",
            "type": "account_warning",
            "name": "account_disabled_spamming",
            "parameters": {"affected_email_address": "bobert@ext.runpanther.io"},
        },
    ),
]


class GSuiteUserSuspended(PantherRule):
    RuleID = "GSuite.UserSuspended-prototype"
    DisplayName = "GSuite User Suspended"
    LogTypes = [PantherLogType.GSuite_ActivityEvent]
    Tags = ["GSuite"]
    Severity = PantherSeverity.High
    Description = (
        "A GSuite user was suspended, the account may have been compromised by a spam network.\n"
    )
    Reference = "https://support.google.com/drive/answer/40695?hl=en&sjid=864417124752637253-EU"
    Runbook = "Investigate the behavior that got the account suspended. Verify with the user that this intended behavior. If not, the account may have been compromised.\n"
    SummaryAttributes = ["actor:email"]
    Tests = g_suite_user_suspended_tests
    USER_SUSPENDED_EVENTS = {
        "account_disabled_generic",
        "account_disabled_spamming_through_relay",
        "account_disabled_spamming",
        "account_disabled_hijacked",
    }

    def rule(self, event):
        if deep_get(event, "id", "applicationName") != "login":
            return False
        return bool(event.get("name") in self.USER_SUSPENDED_EVENTS)

    def title(self, event):
        user = deep_get(event, "parameters", "affected_email_address")
        if not user:
            user = "<UNKNOWN_USER>"
        return f"User [{user}]'s account was disabled"



import json
from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.log_types import PantherLogType

git_lab_audit_password_reset_multiple_emails_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="not a password reset",
        ExpectedResult=False,
        Log={"detail": {"custom_message": "hello world"}},
    ),
    PantherRuleTest(
        Name="one email",
        ExpectedResult=False,
        Log={
            "detail": {
                "custom_message": "Ask for password reset",
                "target_details": "example@test.com",
            }
        },
    ),
    PantherRuleTest(
        Name="multiple emails",
        ExpectedResult=True,
        Log={
            "detail": {
                "custom_message": "Ask for password reset",
                "target_details": '["example@test.com", "example2@test.com"]',
            }
        },
    ),
]


class GitLabAuditPasswordResetMultipleEmails(PantherRule):
    RuleID = "GitLab.Audit.Password.Reset.Multiple.Emails-prototype"
    DisplayName = "CVE-2023-7028 - GitLab Audit Password Reset Multiple Emails"
    LogTypes = [PantherLogType.GitLab_Audit]
    Tags = ["GitLab", "CVE-2023-7028"]
    Reports = {"MITRE ATT&CK": ["TA0001:T1195", "TA0001:T1190", "TA0003:T1098"]}
    Severity = PantherSeverity.High
    Description = "Attackers are exploiting a Critical (CVSS 10.0) GitLab vulnerability in which user account password reset emails could be delivered to an unverified email address."
    Reference = "https://about.gitlab.com/releases/2024/01/11/critical-security-release-gitlab-16-7-2-released/"
    Tests = git_lab_audit_password_reset_multiple_emails_tests

    def rule(self, event):
        custom_message = event.deep_get("detail", "custom_message", default="")
        emails_raw = event.deep_get("detail", "target_details", default="")
        if custom_message != "Ask for password reset":
            return False
        try:
            emails = json.loads(emails_raw)
        except json.decoder.JSONDecodeError:
            return False
        if len(emails) > 1:
            return True
        return False

    def title(self, event):
        emails = event.deep_get("detail", "target_details", default="")
        return f"[GitLab] Multiple password reset emails requested for {emails}"



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.log_types import PantherLogType

g_suite_government_backed_attack_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Normal Login Event",
        ExpectedResult=False,
        Log={
            "id": {"applicationName": "login"},
            "actor": {"email": "homer.simpson@example.com"},
            "type": "login",
            "name": "login_success",
            "parameters": {"is_suspicious": None, "login_challenge_method": ["none"]},
        },
    ),
    PantherRuleTest(
        Name="Government Backed Attack Warning",
        ExpectedResult=True,
        Log={
            "id": {"applicationName": "login"},
            "actor": {"email": "homer.simpson@example.com"},
            "type": "login",
            "name": "gov_attack_warning",
            "parameters": {"is_suspicious": None, "login_challenge_method": ["none"]},
        },
    ),
]


class GSuiteGovernmentBackedAttack(PantherRule):
    RuleID = "GSuite.GovernmentBackedAttack-prototype"
    DisplayName = "GSuite Government Backed Attack"
    LogTypes = [PantherLogType.GSuite_ActivityEvent]
    Tags = ["GSuite"]
    Severity = PantherSeverity.Critical
    Description = (
        "GSuite reported that it detected a government backed attack against your account.\n"
    )
    Reference = "https://support.google.com/a/answer/9007870?hl=en"
    Runbook = "Followup with GSuite support for more details.\n"
    SummaryAttributes = ["actor:email"]
    Tests = g_suite_government_backed_attack_tests

    def rule(self, event):
        if deep_get(event, "id", "applicationName") != "login":
            return False
        return bool(event.get("name") == "gov_attack_warning")

    def title(self, event):
        return f"User [{deep_get(event, 'actor', 'email', default='<UNKNOWN_EMAIL>')}] may have been targeted by a government attack"



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.log_types import PantherLogType

duo_user_endpoint_failure_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="endpoint_is_not_in_management_system",
        ExpectedResult=True,
        Log={
            "access_device": {"ip": "12.12.112.25", "os": "Mac OS X"},
            "auth_device": {"ip": "12.12.12.12"},
            "application": {},
            "event_type": "authentication",
            "factor": "duo_push",
            "reason": "endpoint_is_not_in_management_system",
            "result": "denied",
            "user": {"name": "example@example.io"},
        },
    ),
    PantherRuleTest(
        Name="endpoint_failed_google_verification",
        ExpectedResult=True,
        Log={
            "access_device": {"ip": "12.12.112.25", "os": "Mac OS X"},
            "auth_device": {"ip": "12.12.12.12"},
            "application": {},
            "event_type": "authentication",
            "factor": "duo_push",
            "reason": "endpoint_failed_google_verification",
            "result": "denied",
            "user": {"name": "example@example.io"},
        },
    ),
    PantherRuleTest(
        Name="endpoint_is_not_trusted",
        ExpectedResult=True,
        Log={
            "access_device": {"ip": "12.12.112.25", "os": "Mac OS X"},
            "auth_device": {"ip": "12.12.12.12"},
            "application": {},
            "event_type": "authentication",
            "factor": "duo_push",
            "reason": "endpoint_is_not_trusted",
            "result": "denied",
            "user": {"name": "example@example.io"},
        },
    ),
    PantherRuleTest(
        Name="could_not_determine_if_endpoint_was_trusted",
        ExpectedResult=True,
        Log={
            "access_device": {"ip": "12.12.112.25", "os": "Mac OS X"},
            "auth_device": {"ip": "12.12.12.12"},
            "application": {},
            "event_type": "authentication",
            "factor": "duo_push",
            "reason": "could_not_determine_if_endpoint_was_trusted",
            "result": "denied",
            "user": {"name": "example@example.io"},
        },
    ),
    PantherRuleTest(
        Name="invalid_device",
        ExpectedResult=True,
        Log={
            "access_device": {"ip": "12.12.112.25", "os": "Mac OS X"},
            "auth_device": {"ip": "12.12.12.12"},
            "application": {},
            "event_type": "authentication",
            "factor": "duo_push",
            "reason": "invalid_device",
            "result": "denied",
            "user": {"name": "example@example.io"},
        },
    ),
    PantherRuleTest(
        Name="good_auth",
        ExpectedResult=False,
        Log={
            "access_device": {"ip": "12.12.112.25", "os": "Mac OS X"},
            "auth_device": {"ip": "12.12.12.12"},
            "application": {"key": "D12345", "name": "Slack"},
            "event_type": "authentication",
            "factor": "duo_push",
            "reason": "user_approved",
            "result": "success",
            "user": {"name": "example@example.io"},
        },
    ),
    PantherRuleTest(
        Name="denied_old_creds",
        ExpectedResult=False,
        Log={
            "access_device": {"ip": "12.12.112.25", "os": "Mac OS X"},
            "auth_device": {"ip": "12.12.12.12"},
            "application": {"key": "D12345", "name": "Slack"},
            "event_type": "authentication",
            "factor": "duo_push",
            "reason": "out_of_date",
            "result": "denied",
            "user": {"name": "example@example.io"},
        },
    ),
]


class DUOUserEndpointFailure(PantherRule):
    RuleID = "DUO.User.Endpoint.Failure-prototype"
    DisplayName = "Duo User Denied For Endpoint Error"
    DedupPeriodMinutes = 15
    LogTypes = [PantherLogType.Duo_Authentication]
    Tags = ["Duo"]
    Severity = PantherSeverity.Medium
    Description = "A Duo user's authentication was denied due to a suspicious error on the endpoint"
    Reference = "https://duo.com/docs/adminapi#authentication-logs"
    Runbook = (
        "Follow up with the endpoint owner to see status. Follow up with user to verify attempts."
    )
    Tests = duo_user_endpoint_failure_tests

    def rule(self, event):
        endpoint_reasons = [
            "endpoint_is_not_in_management_system",
            "endpoint_failed_google_verification",
            "endpoint_is_not_trusted",
            "could_not_determine_if_endpoint_was_trusted",
            "invalid_device",
        ]
        return event.get("reason", "") in endpoint_reasons

    def title(self, event):
        user = deep_get(event, "user", "name", default="Unknown")
        reason = event.get("reason", "Unknown")
        return f"Duo User [{user}] encountered suspicious endpoint issue [{reason}]"

    def alert_context(self, event):
        return {
            "factor": event.get("factor"),
            "reason": event.get("reason"),
            "user": deep_get(event, "user", "name", default=""),
            "os": deep_get(event, "access_device", "os", default=""),
            "ip_access": deep_get(event, "access_device", "ip", default=""),
            "ip_auth": deep_get(event, "auth_device", "ip", default=""),
            "application": deep_get(event, "application", "name", default=""),
        }



from typing import List

import pypanther.helpers.panther_event_type_helpers as event_type
from pypanther.base import PantherDataModel, PantherDataModelMapping
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.log_types import PantherLogType

audit_log_type_map = {
    "CREATE_USER": event_type.USER_ACCOUNT_CREATED,
    "DELETE_USER": event_type.USER_ACCOUNT_DELETED,
    "UPDATE_USER": event_type.USER_ACCOUNT_MODIFIED,
    "CREATE_USER_ROLE": event_type.USER_GROUP_CREATED,
    "DELETE_USER_ROLE": event_type.USER_GROUP_DELETED,
    "UPDATE_USER_ROLE": event_type.USER_ROLE_MODIFIED,
}


def get_event_type(event):
    audit_log_type = event.get("actionName")
    matched = audit_log_type_map.get(audit_log_type)
    if matched is not None:
        return matched
    return None


def get_actor_user(event):
    # First prefer actor.attributes.email
    #  automatons like SCIM won't have an actor.attributes.email
    actor_user = deep_get(event, "actor", "attributes", "email")
    if actor_user is None:
        actor_user = deep_get(event, "actor", "id")
    return actor_user


class StandardPantherAudit(PantherDataModel):
    DataModelID: str = "Standard.Panther.Audit"
    DisplayName: str = "Panther Audit Logs"
    Enabled: bool = True
    LogTypes: List[str] = [PantherLogType.Panther_Audit]
    Mappings: List[PantherDataModelMapping] = [
        PantherDataModelMapping(Name="source_ip", Path="sourceIP"),
        PantherDataModelMapping(Name="user_agent", Path="userAgent"),
        PantherDataModelMapping(Name="actor_user", Method=get_actor_user),
        PantherDataModelMapping(Name="user", Path="$.actionParams.input.email"),
        PantherDataModelMapping(Name="event_type", Method=get_event_type),
    ]



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.helpers.panther_base_helpers import gsuite_details_lookup as details_lookup
from pypanther.helpers.panther_base_helpers import gsuite_parameter_lookup as param_lookup
from pypanther.log_types import PantherLogType

g_suite_drive_overly_visible_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Access Event",
        ExpectedResult=False,
        Log={
            "p_row_id": "111222",
            "actor": {"email": "bobert@example.com"},
            "id": {"applicationName": "drive"},
            "events": [{"type": "access", "name": "download"}],
        },
    ),
    PantherRuleTest(
        Name="Modify Event Without Over Visibility",
        ExpectedResult=False,
        Log={
            "p_row_id": "111222",
            "actor": {"email": "bobert@example.com"},
            "id": {"applicationName": "drive"},
            "events": [
                {
                    "type": "access",
                    "name": "edit",
                    "parameters": [{"name": "visibility", "value": "private"}],
                }
            ],
        },
    ),
    PantherRuleTest(
        Name="Overly Visible Doc Modified",
        ExpectedResult=True,
        Log={
            "p_row_id": "111222",
            "actor": {"email": "bobert@example.com"},
            "id": {"applicationName": "drive"},
            "events": [
                {
                    "type": "access",
                    "name": "edit",
                    "parameters": [
                        {"name": "visibility", "value": "people_with_link"},
                        {"name": "doc_title", "value": "my shared document"},
                    ],
                }
            ],
        },
    ),
    PantherRuleTest(
        Name="Overly Visible Doc Modified - no email",
        ExpectedResult=True,
        Log={
            "p_row_id": "111222",
            "actor": {"profileId": "1234567890123"},
            "id": {"applicationName": "drive"},
            "events": [
                {
                    "type": "access",
                    "name": "edit",
                    "parameters": [
                        {"name": "visibility", "value": "people_with_link"},
                        {"name": "doc_title", "value": "my shared document"},
                    ],
                }
            ],
        },
    ),
]


class GSuiteDriveOverlyVisible(PantherRule):
    RuleID = "GSuite.DriveOverlyVisible-prototype"
    DisplayName = "GSuite Overly Visible Drive Document"
    LogTypes = [PantherLogType.GSuite_Reports]
    Tags = ["GSuite", "Collection:Data from Information Repositories"]
    Reports = {"MITRE ATT&CK": ["TA0009:T1213"]}
    Severity = PantherSeverity.Info
    Description = "A Google drive resource that is overly visible has been modified.\n"
    Reference = "https://support.google.com/docs/answer/2494822?hl=en&co=GENIE.Platform%3DDesktop&sjid=864417124752637253-EU"
    Runbook = "Investigate whether the drive document is appropriate to be this visible.\n"
    SummaryAttributes = ["actor:email"]
    DedupPeriodMinutes = 360
    Tests = g_suite_drive_overly_visible_tests
    RESOURCE_CHANGE_EVENTS = {"create", "move", "upload", "edit"}
    PERMISSIVE_VISIBILITY = {"people_with_link", "public_on_the_web"}

    def rule(self, event):
        if deep_get(event, "id", "applicationName") != "drive":
            return False
        details = details_lookup("access", self.RESOURCE_CHANGE_EVENTS, event)
        return (
            bool(details)
            and param_lookup(details.get("parameters", {}), "visibility")
            in self.PERMISSIVE_VISIBILITY
        )

    def dedup(self, event):
        user = deep_get(event, "actor", "email")
        if user is None:
            user = deep_get(event, "actor", "profileId", default="<UNKNOWN_PROFILEID>")
        return user

    def title(self, event):
        details = details_lookup("access", self.RESOURCE_CHANGE_EVENTS, event)
        doc_title = param_lookup(details.get("parameters", {}), "doc_title")
        share_settings = param_lookup(details.get("parameters", {}), "visibility")
        user = deep_get(event, "actor", "email")
        if user is None:
            user = deep_get(event, "actor", "profileId", default="<UNKNOWN_PROFILEID>")
        return f"User [{user}] modified a document [{doc_title}] that has overly permissive share settings [{share_settings}]"



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import m365_alert_context
from pypanther.log_types import PantherLogType

microsoft365_brute_force_loginby_user_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Failed Login event",
        ExpectedResult=True,
        Log={
            "Actor": [
                {"ID": "012345-abcde-543-xyz", "Type": 0},
                {"ID": "sample.user@yourorg.onmicrosoft.com", "Type": 5},
            ],
            "ActorContextId": "123-abc-xyz-567",
            "ActorIpAddress": "1.2.3.4",
            "ApplicationId": "123-abc-sfa-321",
            "AzureActiveDirectoryEventType": 1,
            "ClientIP": "1.2.3.4",
            "CreationTime": "2022-12-12 15:57:57",
            "ExtendedProperties": [
                {"Name": "ResultStatusDetail", "Value": "Success"},
                {
                    "Name": "UserAgent",
                    "Value": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/107.0.0.0 Safari/537.36",
                },
                {"Name": "UserAuthenticationMethod", "Value": "1"},
                {"Name": "RequestType", "Value": "Login:login"},
            ],
            "Id": "abc-def-123",
            "InterSystemsId": "987-432-123",
            "IntraSystemId": "aaa-bbb-ccc",
            "LogonError": "InvalidUserNameOrPassword",
            "ObjectId": "aa-11-22-bb",
            "Operation": "UserLoginFailed",
            "OrganizationId": "11-aa-22-bb",
            "RecordType": 15,
            "ResultStatus": "Success",
            "SupportTicketId": "",
            "Target": [{"ID": "11-22-33", "Type": 0}],
            "TargetContextId": "11-22-33-44",
            "UserId": "sample.user@yourorg.onmicrosoft.com",
            "UserKey": "012345-abcde-543-xyz",
            "UserType": 0,
            "Workload": "AzureActiveDirectory",
        },
    ),
    PantherRuleTest(
        Name="Login Event",
        ExpectedResult=False,
        Log={
            "Actor": [
                {"ID": "012345-abcde-543-xyz", "Type": 0},
                {"ID": "sample.user@yourorg.onmicrosoft.com", "Type": 5},
            ],
            "ActorContextId": "123-abc-xyz-567",
            "ActorIpAddress": "1.2.3.4",
            "ApplicationId": "123-abc-sfa-321",
            "AzureActiveDirectoryEventType": 1,
            "ClientIP": "1.2.3.4",
            "CreationTime": "2022-12-12 15:57:57",
            "ExtendedProperties": [
                {"Name": "ResultStatusDetail", "Value": "Success"},
                {
                    "Name": "UserAgent",
                    "Value": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/107.0.0.0 Safari/537.36",
                },
                {"Name": "RequestType", "Value": "Login:reprocess"},
            ],
            "Id": "abc-def-123",
            "InterSystemsId": "987-432-123",
            "IntraSystemId": "aaa-bbb-ccc",
            "ObjectId": "aa-11-22-bb",
            "Operation": "UserLoggedIn",
            "OrganizationId": "11-aa-22-bb",
            "RecordType": 15,
            "ResultStatus": "Success",
            "SupportTicketId": "",
            "Target": [{"ID": "11-22-33", "Type": 0}],
            "TargetContextId": "11-22-33-44",
            "UserId": "sample.user@yourorg.onmicrosoft.com",
            "UserKey": "012345-abcde-543-xyz",
            "UserType": 0,
        },
    ),
]


class Microsoft365BruteForceLoginbyUser(PantherRule):
    Description = "A Microsoft365 user was denied login access several times"
    DisplayName = "Microsoft365 Brute Force Login by User"
    Reports = {"MITRE ATT&CK": ["TA0006:T1110"]}
    Runbook = "Analyze the IP they came from and actions taken before/after."
    Reference = "https://learn.microsoft.com/en-us/microsoft-365/troubleshoot/authentication/access-denied-when-connect-to-office-365"
    Severity = PantherSeverity.Medium
    LogTypes = [PantherLogType.Microsoft365_Audit_AzureActiveDirectory]
    RuleID = "Microsoft365.Brute.Force.Login.by.User-prototype"
    Threshold = 10
    Tests = microsoft365_brute_force_loginby_user_tests

    def rule(self, event):
        return event.get("Operation", "") == "UserLoginFailed"

    def title(self, event):
        return f"Microsoft365: [{event.get('UserId', '<user-not-found>')}] may be undergoing a Brute Force Attack."

    def alert_context(self, event):
        return m365_alert_context(event)



from typing import List

from pypanther.base import PantherDataModel, PantherDataModelMapping
from pypanther.log_types import PantherLogType


class StandardOCSFDnsActivity(PantherDataModel):
    DataModelID: str = "Standard.OCSF.DnsActivity"
    DisplayName: str = "OCSF DNS Activity"
    Enabled: bool = True
    LogTypes: List[str] = [PantherLogType.OCSF_DnsActivity]
    Mappings: List[PantherDataModelMapping] = [
        PantherDataModelMapping(Name="source_ip", Path="$.src_endpoint.ip"),
        PantherDataModelMapping(Name="source_port", Path="$.src_endpoint.port"),
        PantherDataModelMapping(Name="dns_query", Path="$.query.hostname"),
    ]



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.log_types import PantherLogType

osquery_mac_auto_update_enabled_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Auto Updates Disabled",
        ExpectedResult=True,
        Log={
            "columns": {
                "domain": "com.apple.SoftwareUpdate",
                "key": "AutomaticCheckEnabled",
                "value": "false",
            },
            "action": "added",
            "name": "pack/mac-cis/SoftwareUpdate",
        },
    ),
    PantherRuleTest(
        Name="Auto Updates Enabled",
        ExpectedResult=False,
        Log={
            "columns": {
                "domain": "com.apple.SoftwareUpdate",
                "key": "AutomaticCheckEnabled",
                "value": "true",
            },
            "action": "added",
            "name": "pack/mac-cis/SoftwareUpdate",
        },
    ),
    PantherRuleTest(
        Name="Wrong Key",
        ExpectedResult=False,
        Log={
            "columns": {
                "domain": "com.apple.SoftwareUpdate",
                "key": "LastFullSuccessfulDate",
                "value": "false",
            },
            "action": "added",
            "name": "pack/mac-cis/SoftwareUpdate",
        },
    ),
]


class OsqueryMacAutoUpdateEnabled(PantherRule):
    RuleID = "Osquery.Mac.AutoUpdateEnabled-prototype"
    DisplayName = "OSQuery Reports Application Firewall Disabled"
    LogTypes = [PantherLogType.Osquery_Differential]
    Tags = ["Osquery", "MacOS", "Security Control", "Defense Evasion:Impair Defenses"]
    Reports = {"CIS": ["1.2"], "MITRE ATT&CK": ["TA0005:T1562"]}
    Severity = PantherSeverity.Medium
    DedupPeriodMinutes = 1440
    Description = "Verifies that MacOS has automatic software updates enabled.\n"
    Runbook = "Enable the auto updates on the host.\n"
    Reference = "https://support.apple.com/en-gb/guide/mac-help/mchlpx1065/mac"
    SummaryAttributes = ["name", "action", "p_any_ip_addresses", "p_any_domain_names"]
    Tests = osquery_mac_auto_update_enabled_tests

    def rule(self, event):
        # Send an alert if not set to "true"
        return (
            "SoftwareUpdate" in event.get("name", [])
            and event.get("action") == "added"
            and (deep_get(event, "columns", "domain") == "com.apple.SoftwareUpdate")
            and (deep_get(event, "columns", "key") == "AutomaticCheckEnabled")
            and (deep_get(event, "columns", "value") == "false")
        )



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.log_types import PantherLogType

git_hub_user_role_updated_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="GitHub - Member Updated",
        ExpectedResult=True,
        Log={
            "actor": "cat",
            "action": "org.update_member",
            "created_at": 1621305118553,
            "p_log_type": "GitHub.Audit",
            "repo": "my-org/my-repo",
            "user": "bob",
        },
    ),
    PantherRuleTest(
        Name="GitHub - Member Invited",
        ExpectedResult=False,
        Log={
            "actor": "cat",
            "action": "org.invite_member",
            "created_at": 1621305118553,
            "p_log_type": "GitHub.Audit",
            "repo": "my-org/my-repo",
            "user": "bob",
        },
    ),
]


class GitHubUserRoleUpdated(PantherRule):
    RuleID = "GitHub.User.RoleUpdated-prototype"
    DisplayName = "GitHub User Role Updated"
    LogTypes = [PantherLogType.GitHub_Audit]
    Tags = ["GitHub", "Persistence:Account Manipulation"]
    Reports = {"MITRE ATT&CK": ["TA0003:T1098"]}
    Reference = "https://docs.github.com/en/organizations/managing-peoples-access-to-your-organization-with-roles/roles-in-an-organization"
    Severity = PantherSeverity.High
    Description = (
        "Detects when a GitHub user role is upgraded to an admin or downgraded to a member"
    )
    Tests = git_hub_user_role_updated_tests

    def rule(self, event):
        return event.get("action") == "org.update_member"

    def title(self, event):
        return f"Org owner [{event.udm('actor_user')}] updated user's [{event.get('user')}] role ('admin' or 'member')"



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.log_types import PantherLogType

one_login_user_account_locked_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="User account locked via api - first method.",
        ExpectedResult=True,
        Log={
            "event_type_id": "531",
            "actor_user_id": 123456,
            "actor_user_name": "Bob Cat",
            "user_id": 123456,
            "user_name": "Bob Cat",
        },
    ),
    PantherRuleTest(
        Name="User account locked via api - second method.",
        ExpectedResult=True,
        Log={
            "event_type_id": "553",
            "actor_user_id": 654321,
            "actor_user_name": "Mountain Lion",
            "user_id": 123456,
            "user_name": "Bob Cat",
        },
    ),
    PantherRuleTest(
        Name="User account suspended via api.",
        ExpectedResult=True,
        Log={
            "event_type_id": "551",
            "actor_user_id": 654321,
            "actor_user_name": "Mountain Lion",
            "user_id": 123456,
            "user_name": "Bob Cat",
        },
    ),
    PantherRuleTest(
        Name="Normal User Activated Event",
        ExpectedResult=False,
        Log={
            "event_type_id": "11",
            "actor_user_id": 654321,
            "actor_user_name": "Mountain Lion",
            "user_id": 123456,
            "user_name": "Bob Cat",
        },
    ),
]


class OneLoginUserAccountLocked(PantherRule):
    RuleID = "OneLogin.UserAccountLocked-prototype"
    DisplayName = "OneLogin User Locked"
    LogTypes = [PantherLogType.OneLogin_Events]
    Tags = ["OneLogin", "Credential Access:Brute Force"]
    Reports = {"MITRE ATT&CK": ["TA0006:T1110"]}
    Severity = PantherSeverity.Low
    Description = "User locked or suspended from their account.\n"
    Reference = "https://onelogin.service-now.com/kb_view_customer.do?sysparm_article=KB0010420"
    Runbook = "Investigate whether this was caused by expected action.\n"
    SummaryAttributes = ["account_id", "event_type_id", "user_name", "user_id"]
    Tests = one_login_user_account_locked_tests

    def rule(self, event):
        # check for a user locked event
        # event 531 and 553 are user lock events via api
        # event 551 is user suspended via api
        return str(event.get("event_type_id")) in ["531", "553", "551"]

    def title(self, event):
        return f"A user [{event.get('user_name', '<UNKNOWN_USER>')}] was locked or suspended via api call"
